/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    ctr_process.sas
| PRODUCT: SAS Currency Transaction Reporting
| SYSTEM:  Windows, Unix
| DESC:    Transform AML data into CTR data 
+------------------------------------------------------------------------------+
| USAGE:   %ctr_process
| Assumption: there is only one person in the alert
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

 

%macro ctr_process(  
                        inIncidentDsn=ctr_incident,
                        inTxnDsn=triggering_txn,
                        inPartyDsn=party_dim,                        
                        inBranchDsn=x_branch,
                        inFiler=x_filer,
                        testYN=N 

                       )

       ;

    /*------------
     * Set Module
     *------------*/
    %let _fcfmodule_=CTR_PROCESS;

    /*-----------------------------
     * Set start time for jobstats
     *-----------------------------*/
    %fcf_section_header(section=&_fcfmodule_, message_format=verbose);


    %let trans_rc=0;
    %let _fcfmsg_=;


 %ctr_custom_init; /* Execute any custom code before transforming FCF data into CTR data */

  
 /*-----------------------------
  * Define CTR transaction data  
  *-----------------------------*/
      
%let cash_in_dim=10;
%let cash_out_dim=11;
data ctr 
     foreign_cash_in_txn (keep=alert_id transaction_currency_key  currency_amount_in_txn_ccy)
     foreign_cash_out_txn (keep=alert_id transaction_currency_key  currency_amount_in_txn_ccy)
     ;
     set &inTxnDsn;
     by alert_id;
     length X_US_CASH_IN_OTH_TXT 
            X_US_CASH_OUT_OTH_TXT
            $  50
     ;
     retain x_us_cash_in_oth_txt x_us_cash_out_oth_txt ' '; 
     retain X_US_CASH_IN_AMT 
            X_US_CASH_OUT_AMT 
            0
    ;
    retain X_TRANSACTION_DT 8;
    format X_TRANSACTION_DT nldate.;
    retain txn_cnt 0;
    length X_TRAN_METHOD_TYPE_LIST $ 6; 
    retain x_TRAN_METHOD_TYPE_LIST '000010';  /* assuming aggregated transactions initially */
    length X_PERSON_INVOLVED_TYPE_CD $ 1;
    retain X_PERSON_INVOLVED_TYPE_CD;
     array us_cash_in_amt_by_type(&cash_in_dim) 8 
            X_US_CASH_IN_A_AMT
            X_US_CASH_IN_B_AMT
            X_US_CASH_IN_C_AMT
            X_US_CASH_IN_D_AMT
            X_US_CASH_IN_E_AMT
            X_US_CASH_IN_F_AMT 
            X_US_CASH_IN_G_AMT 
            X_US_CASH_IN_H_AMT
            X_US_CASH_IN_I_AMT
            X_US_CASH_IN_Z_AMT
        
           ;
     
     array us_cash_out_amt_by_type(&cash_out_dim) 8 
            X_US_CASH_OUT_A_AMT
            X_US_CASH_OUT_B_AMT
            X_US_CASH_OUT_C_AMT
            X_US_CASH_OUT_D_AMT
            X_US_CASH_OUT_E_AMT
            X_US_CASH_OUT_F_AMT 
            X_US_CASH_OUT_G_AMT 
            X_US_CASH_OUT_H_AMT
            X_US_CASH_OUT_I_AMT
            X_US_CASH_OUT_J_AMT
            X_US_CASH_OUT_Z_AMT
            ;
   
     retain cash_in_type_list  'ABCDEFGHIZ';
     retain cash_out_type_list 'ABCDEFGHIJZ';
      
     ;
    
     drop cash_in_type_list cash_out_type_list txn_cnt calendar_date_sas i; 
     
     if first.alert_id then link INIT_ALERT; 
      

     %ctr_custom_cash_type;

     %ctr_custom_tran_method_type;
  
     if transaction_currency_key ne &us_currency_key then
     do;
         if transaction_cdi_code='C' then output foreign_cash_in_txn;
         else if transaction_cdi_code='D' then output foreign_cash_out_txn;
     end;

     if transaction_cdi_code='C' then
     do; 
        us_cash_in_amt=currency_amount;
        us_cash_in_amt_by_type{indexc(cash_in_type_list,cash_in_type)}+us_cash_in_amt;
     end;
     else if transaction_cdi_code='D' then
     do;
          us_cash_out_amt=currency_amount; 
          us_cash_out_amt_by_type{indexc(cash_out_type_list,cash_out_type)}+us_cash_out_amt;
     end;
     txn_cnt+1;

     if ceil(us_cash_in_amt) > 10000 or ceil(us_cash_out_amt) > 10000 then substr(x_tran_method_type_list,5,1)='0';

     /* Assuming the scenario is triggered by account owners */
     /* Will Change when it is triggered by executing parties */
     /* If the account owner is conducting the transaction, executing_party_key is -1 */
     
     if executing_party_key=-1 then X_PERSON_INVOLVED_TYPE_CD='A'; 
     

     if last.alert_id then link TERM_ALERT;    
     
     return;

INIT_ALERT:

    do i=1 to &cash_in_dim;
       us_cash_in_amt_by_type{i}=.;
       
    end;
    do i=1 to &cash_out_dim;
       us_cash_out_amt_by_type{i}=.;
       
    end;

    x_us_cash_in_oth_txt='';
    x_us_cash_out_oth_txt='';
    x_us_cash_in_amt=.;
    x_us_cash_out_amt=.; 
    x_transaction_dt=calendar_date_sas;
    txn_cnt=0;
    X_PERSON_INVOLVED_TYPE_CD='C'; /* start with 'Person on whose behalf transaction is conducted' */

return;

TERM_ALERT:

    do i=1 to &cash_in_dim;
       if us_cash_in_amt_by_type{i} > 0 then
       do; 
          us_cash_in_amt_by_type{i}=ceil(us_cash_in_amt_by_type{i});
          x_us_cash_in_amt+us_cash_in_amt_by_type{i};
       end;
    end;
    do i=1 to &cash_out_dim;
       if us_cash_out_amt_by_type{i} > 0 then 
       do;
         us_cash_out_amt_by_type{i}=ceil(us_cash_out_amt_by_type{i});
         x_us_cash_out_amt+us_cash_out_amt_by_type{i};
       end;
    end;

    if txn_cnt > 1 then X_MULTI_TRANSACTION_FLG='1';
    else X_MULTI_TRANSACTION_FLG='0';

    output ctr;
 
return;

run;

%fcf_rcset(&syserr);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;



  
/*-----------------------------------------------------
 * Define CTR person account data (X_ACCOUNT_CASH_IN
 * and X_ACCOUNT_CASH_OUT) 
 *----------------------------------------------------*/
proc sql;
    create table x_account_cash_in as
    select distinct alert_id as RR_RK, 
                    1 as X_PERSON_INVOLVED_RK, 
                    compress(account_number,,'ADK') as X_CASH_IN_ACCT_ID
    from &inTxnDsn
    where transaction_cdi_code='C'
    order by rr_rk;

%fcf_rcset(&sqlrc);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;



    create table x_account_cash_out as
    select distinct alert_id as RR_RK, 
                    1 as X_PERSON_INVOLVED_RK, 
                    compress(account_number,,'ADK') as X_CASH_OUT_ACCT_ID
    from &inTxnDsn
    where transaction_cdi_code='D'
    order by rr_rk;
    
%fcf_rcset(&sqlrc);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;



  
/*-------------------------------------------
 * Define CTR Branch Data (X_INSTITUTION) 
 *------------------------------------------*/

  
 %ctr_custom_branch(inTxnDsn=&inTxnDsn,inBranchDsn=x_branch);

%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;


  
/*----------------------------------------------------------
 * Define CTR Personal Involved Data (X_PERSON_INVOLVED) 
 *---------------------------------------------------------*/

proc sql;

    create table x_person_involved as 
    select          ctr.alert_id as RR_RK,
                    ifc(pty.party_type_desc='INDIVIDUAL',ctr.X_PERSON_INVOLVED_TYPE_CD,'C') as X_PERSON_INVOLVED_TYPE_CD,
                    related_pty.party_rk as X_PARTY_RK, 
                    1 as X_PERSON_INVOLVED_RK,
                    ctr.X_MULTI_TRANSACTION_FLG,
                    ctr.X_US_CASH_IN_AMT as X_CASH_IN_AMT,
                    ctr.X_US_CASH_OUT_AMT as X_CASH_OUT_AMT,
                    %ctr_custom_party             
             
    from  ctr             as ctr,
          &inPartyDsn     as pty,
          &inIncidentDsn  as related_pty

    where ctr.party_key=pty.party_key 
          and ctr.alert_id=related_pty.alert_id
     order by rr_rk
     ;

 %fcf_rcset(&sqlrc);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;

quit;
  
/*-----------------------------
 * Define RR table 
 *-----------------------------*/


data rr;
     merge ctr     (keep=ALERT_ID                    
                    X_TRANSACTION_DT
                    X_US_CASH_IN_AMT
                    X_US_CASH_OUT_AMT
                    X_US_CASH_IN_A_AMT
                    X_US_CASH_IN_B_AMT
                    X_US_CASH_IN_C_AMT
                    X_US_CASH_IN_D_AMT
                    X_US_CASH_IN_E_AMT
                    X_US_CASH_IN_F_AMT 
                    X_US_CASH_IN_G_AMT 
                    X_US_CASH_IN_H_AMT
                    X_US_CASH_IN_I_AMT
                    X_US_CASH_IN_Z_AMT
                    X_US_CASH_IN_OTH_TXT
                    X_US_CASH_OUT_A_AMT
                    X_US_CASH_OUT_B_AMT
                    X_US_CASH_OUT_C_AMT
                    X_US_CASH_OUT_D_AMT
                    X_US_CASH_OUT_E_AMT
                    X_US_CASH_OUT_F_AMT 
                    X_US_CASH_OUT_G_AMT 
                    X_US_CASH_OUT_H_AMT
                    X_US_CASH_OUT_I_AMT
                    X_US_CASH_OUT_J_AMT
                    X_US_CASH_OUT_Z_AMT
                    X_US_CASH_OUT_OTH_TXT          
                    )
           ctr_incident (keep=alert_id incident_rk)
           ;
        by alert_id;

%ctr_custom_filer;

rename alert_id=RR_RK;

RR_DESC='Auto CTR Alert '||strip(put(alert_id,best.));
RR_TYPE_CD='US';
RR_CATEGORY_CD='FINCEN';
RR_SUBCATEGORY_CD='CTRX';
X_CTR_FILING_TYPE_CD='A'; /* initial filing */
run;

%fcf_rcset(&syserr);

%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;


/*--------------------------------------------- 
 * Define Transaction methods (X_TRAN_METHOD)
 *-------------------------------------------*/


data x_tran_method;
set ctr;
rename alert_id=RR_RK;
method_list='ABCDEF';
keep alert_id x_tran_method_type_cd;
do i=1 to 6;
   if substr(x_tran_method_type_list,1,i)='1' then 
   do;
      x_tran_method_type_cd=substr(method_list,i,1); 
      output;
   end;
end;
run;

%fcf_rcset(&syserr);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;


/*----------------------------------------------------------------------- 
 * Define Foreign Transactions (X_FOREIGN_CASH_IN and X_FOREIGN_CASH_OUT 
 *-----------------------------------------------------------------------*/

proc sql ;
      create table x_foreign_cash_in as
      select alert_id as RR_RK,
             ifc(substr(ccy.currency_code,1,2)='EU', 'BE', substr(ccy.currency_code,1,2)) as  X_FOREIGN_CASH_IN_COUNTRY_CD,
             ceil(currency_amount_in_txn_ccy) as X_FOREIGN_CASH_IN_AMT
      from foreign_cash_in_txn as txn,
           seg_kc.fsc_currency_dim as ccy
      where txn.transaction_currency_key=ccy.currency_key
      ;
      create table x_foreign_cash_out as
      select alert_id as RR_RK,
             ifc(substr(ccy.currency_code,1,2)='EU', 'BE', substr(ccy.currency_code,1,2)) as  X_FOREIGN_CASH_OUT_COUNTRY_CD,
             ceil(currency_amount_in_txn_ccy) as X_FOREIGN_CASH_OUT_AMT
      from foreign_cash_out_txn as txn,
           seg_kc.fsc_currency_dim as ccy
      where txn.transaction_currency_key=ccy.currency_key
      ;
quit;
 
%fcf_rcset(&sqlrc);
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;




%ctr_custom_term; /* Execute any custom code before creating the reports in ECM */


%ctr_create_rr(testYN=&testYN);

 
%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
%end;


%MACRO_END:
 
    /* Set end time and calculate elapsed processing time for jobstats */
    %fcf_section_footer(section=&_fcfmodule_, message_format=verbose);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
       %fcf_execute_macro(fcf_error);
    %else %do;
       %fcf_execute_macro(fcf_success);
    %end; 
%mend ctr_process;
