/* MAINDOC
+----------------------------------------------------------------------------------------------+
| NAME:    ctr_custom_macro.sas
| PRODUCT: SAS Currency Transaction Reporting
| SYSTEM:  Windows, Unix
| DESC:    Define custom macro for ctr_process.sas 
|          This macro should be adjusted by the site. 
|          
+---------------------------------------------------------------------------------------------+
| USAGE:   %ctr_custom_macro;
+---------------------------------------------------------------------------------------------+
MAINDOCEND */

/*---------------------------------------------------------------------------------------------
 * Copyright (c) 2015-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *--------------------------------------------------------------------------------------------*/

%macro ctr_custom_macro; 

%macro ctr_custom_init;

/* Macro to be executed before converting FCF data into CTR in ctr_process.sas */

%mend;

%macro ctr_custom_cash_type;
  
    /* Data step code to define cash in or cash out type.                    */
    /* Refer to CTR requirement for the meaning of each cash type            */
    /* x_us_cash_in_oth_txt is required if the cash-in type is others (Z)    */
    /* x_us_cash_out_oth_txt is required if the cash-out type is others (Z)  */


 
     if transaction_cdi_code='C' then
    do;
       select;
           when ( transaction_currency_key ne &us_currency_key) cash_in_type='E';
           when ( PRIMARY_MEDIUM_DESC in ("CASH" "CHECK")) cash_in_type='A';
           otherwise  
           do;
                 cash_in_type='Z';
                 if index(x_us_cash_in_oth_txt,strip(primary_medium_desc)) <= 0 then
                    x_us_cash_in_oth_txt=strip(x_us_cash_in_oth_txt)||' '||strip(primary_medium_desc);
           end;
       end; 
    end;
    else if transaction_cdi_code='D' then
    do;
       select ;
           when ( transaction_currency_key ne &us_currency_key) cash_out_type='E';
           when (PRIMARY_MEDIUM_DESC in ("CASH" "CHECK")) cash_out_type='A';
           otherwise
           do;
                  cash_out_type='Z';
                  if index(x_us_cash_out_oth_txt,strip(primary_medium_desc)) <=0  then 
                      x_us_cash_out_oth_txt=strip(x_us_cash_out_oth_txt)||' '||strip(primary_medium_desc);
           end;
       end; 
    end;
      
      
     
%mend;

%macro ctr_custom_tran_method_type;

     /* SAS data step code to define transaction method                              */
     /* X_TRANS_METHOD_LIST is a 6 bit strings to represent 6 different method types */
     /* Refer to the CTR requirement for the meaning of each type.                   */
     /* Change the bit string to '1' for the position where the method type is       */
     /* represented.                                                                 */
     /* 1: Armored Car (FI contact)                                                  */
     /* 2: ATM                                                                       */
     /* 3: Mail Deposit or Shipment                                                  */
     /* 4: Night Deposit                                                             */
     /* 5: Aggregated transactions (this bit will be set to '0' by ctr_process.sas   */
     /*    if $10K+ transactions are found.                                          */ 
     /* 6: Shared branching                                                          */
   

     if mechanism_desc='ATM' then substr(x_trans_method_list,2,1)='1';
     
     
%mend;

%macro ctr_extract_filer_data( institution_id_list=
                              ,outdsn=X_INSTITUTION);

    %let inst_date_where=%str(today() between x_institution_open_dt and x_institution_close_dt);
    %if %sysfunc(exist(&outdsn)) %then 
    %do;
        proc datasets nolist nowarn lib=work;
             delete &outdsn;
        quit;
    %end;

    %ecm_generic_data_get(   udf_table_nm=x_institution
                           , udf_nm=x_institution_id
                           , udf_value_table_nm=generic_data_udf_char_value
                           , udf_value_list=&institution_id_list                                  
                           , where_cl=%superq(inst_date_where)
                           , outdsn=&outdsn
                                     
                         );

      %if not %sysfunc(exist(&outdsn)) %then 
              
      %fcf_rcset(8);
    
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: CTR: Cannot retrieve institution data.;
          %fcf_put_error;
         
     %end;
     
   

 %mend;
  
   

 %macro ctr_custom_filer;
    
    if _n_=1 then 
    do;
        set x_filer (keep=generic_data_rk
                          x_fed_regulator_cd
                          x_institution_type_cd
                          x_institution_nm
                          x_institution_address_txt
                          x_institution_city_nm
                          x_institution_state_cd
                          x_institution_zip_cd
                          x_institution_id
                          x_contact_office_nm
                          x_contact_phone_no
                          x_contact_phone_ext_no
                    )
                          ;
 
        X_INSTITUTION_RK=generic_data_rk;
        /****************************************************************************/
        /* CTR and SAR do not share the same list of regulator and type codes.      */   
        /* The following example have hardcoded regulatory and type code            */
        /* that should be changed to meet your site need.                           */
        /****************************************************************************/
        X_FILING_INST_PRIM_FED_REG_CD='A';
        X_FILING_INST_TYPE_CTR_CD= 'B';
        X_FILING_INST_TYPE_OTH_TXT= '';
        X_FILING_INST_GAMING_TYPE_CD= '';
        X_FILING_INST_GAMING_TYPE_OTH= '';
        
        X_FILING_INST_NM=x_institution_nm;
        X_FILING_INST_ADDRESS_TXT=X_INSTITUTION_ADDRESS_TXT;
        X_FILING_INST_CITY_NM=x_institution_city_nm;
        X_FILING_INST_STATE_CD=x_institution_state_cd;
        X_FILING_INST_COUNTRY_CD='US';
        X_FILING_INST_POSTAL_CD=x_institution_zip_cd;
        X_FILING_INST_TIN_NO=x_institution_id;
        X_FINCEN_TRNSMTTR_TCC='';
        X_CTR_CONTACT_OFFICE=x_contact_office_nm;
        X_CTR_CONTACT_PHONE_NO=x_contact_phone_no;
        X_CTR_CONTACT_PHONE_EXT_NO=x_contact_phone_ext_no;
         
        retain  X_INSTITUTION_RK   
                X_FILING_INST_PRIM_FED_REG_CD
                X_FILING_INST_TYPE_CTR_CD
                X_FILING_INST_TYPE_OTH_TXT
                X_FILING_INST_GAMING_TYPE_CD
                X_FILING_INST_GAMING_TYPE_OTH
                X_FILING_INST_NM
                X_FILING_INST_ADDRESS_TXT
                X_FILING_INST_CITY_NM
                X_FILING_INST_STATE_CD
                X_FILING_INST_COUNTRY_CD
                X_FILING_INST_POSTAL_CD
                X_FILING_INST_TIN_NO
                X_FINCEN_TRNSMTTR_TCC
                X_CTR_CONTACT_OFFICE
                X_CTR_CONTACT_PHONE_NO
                X_CTR_CONTACT_PHONE_EXT_NO
        ;
    end;
    drop generic_data_rk
         x_fed_regulator_cd
         x_institution_nm
         x_institution_address_txt
         x_institution_city_nm
         x_institution_state_cd
         x_institution_zip_cd
         x_institution_id
         x_contact_office_nm
         x_contact_phone_no
         x_contact_phone_ext_no
          ;
%mend;

%macro ctr_custom_party;

               /* SQL query to map the party_dim data to CTR Person Involved data   */
               /* FinCEN has different requirements for entities and individuals.   */
               /* It is important to mark the unknown flag based on PARTY_TYPE_DESC */ 
                ifc(pty.party_type_desc='ORGANIZATION','1','0') as X_ENTITY_FLG,
                ifc(pty.party_type_desc='ORGANIZATION', compress(party_name,'$'), 
                                                        compress(party_last_name,'$') 
                    )  AS X_LAST_NM,
                ifc(pty.party_type_desc='ORGANIZATION', ifc(compress(party_name,'$')='','1','0'), 
                                                        ifc(compress(party_last_name,'$')='','1','0')
                    ) as X_LAST_NM_UNKNOWN_FLG, 
                ifc(pty.party_type_desc='ORGANIZATION', doing_business_as_name, '')  AS X_ALTERNATE_NM,
 
                ifc(pty.party_type_desc ne 'ORGANIZATION', compress(party_first_name,'$'), '') as X_FIRST_NM,
                ifc(pty.party_type_desc ne 'ORGANIZATION', ifc(compress(party_first_name,'$')='','1','0'),
                                                           ''
                    ) as X_FIRST_NM_UNKNOWN_FLG,   
                ifc(pty.party_type_desc ne 'ORGANIZATION', compress(party_middle_name,'$'),'') AS X_MIDDLE_NM,
                               
                strip(compress(pty.street_address_1,'$'))||' '||strip(compress(street_address_2,'$')) as X_ADDRESS_TXT length=100,
                ifc(compress(pty.street_address_1,'$')="" and compress(street_address_2,'$')="",'1','0') as X_ADDRESS_UNKNOWN_FLG,
                compress(pty.street_city_name,'$') as X_CITY_NM,
                ifc(compress(pty.street_city_name,'$')='','1','0') as X_CITY_UNKNOWN_FLG,
                ifc(put(compress(pty.street_country_code,'$'),$_na_ctry_3_fmt.)='Y' and compress(pty.street_state_code,'$') ne '',
                                                             pty.street_state_code,''
                    )  as X_STATE_CD,
                ifc(compress(pty.street_state_code,'$')="",ifc(put(pty.street_country_code,$_na_ctry_3_fmt.)='Y','1',''),
                                             ''
                    ) as X_STATE_UNKNOWN_FLG,                   
                compress(pty.street_postal_code,'$') as X_POSTAL_CD,
                ifc(compress(pty.street_postal_code,'$')='','1','0') as X_POSTAL_UNKNOWN_FLG,
                put(compress(pty.street_country_code,'$'),$_ctry_cd_3to2_fmt.) as X_COUNTRY_CD length=2,
                ifc(compress(pty.street_country_code,'$')="",'1','0') as X_COUNTRY_UNKNOWN_FLG,
                compress(pty.party_tax_id,'$') as X_TIN_NO,
                ifc(compress(pty.party_tax_id,'$')="",'1','0') as X_TIN_UNKNOWN_FLG,
                put(pty.party_tax_id_type_code,$_tax_id_type.) as X_TIN_TYPE_CD,
                ifc(pty.party_type_desc ne 'ORGANIZATION',ifc(pty.party_date_of_birth ne .,
                                                              put(datepart(pty.party_date_of_birth),mmddyyn8.),
                                                              ''
                                                              ),
                                                         ''
                    ) as X_BIRTHDAY_TXT,  
                ifc(pty.party_type_desc ne 'ORGANIZATION', ifc(pty.party_date_of_birth = .,'1','0'),
                                                           ''
                    ) as X_BIRTH_DT_UNKNOWN_FLG,
                compress(pty.phone_number_1,'','kd') as X_PHONE_NO,
                compress(pty.occupation_desc,'$') as X_OCCUPATION_TXT,
                substrn(pty.industry_code_rr,1,6) as X_NAICS_CD,
                ifc(pty.party_type_desc ='ORGANIZATION',ifc(compress(pty.party_identification_type_desc,'$') ='','A','Z'),
                                                        put(compress(pty.party_identification_type_desc,'$'),$_personal_id_type.)
                    ) as X_PERSONAL_ID_TYPE_CD,
                compress(pty.party_identification_id,'$') as X_ID_NO,
                put(compress(pty.party_identification_type_desc,'$'),$_personal_other_id_desc.) as X_PERSONAL_ID_TYPE_OTH_TXT,
                ifc(put(compress(pty.party_id_country_code,'$'),$_na_ctry_3_fmt.)='Y',compress(pty.party_id_state_code,'$'), '')  as X_ID_AUTHORITY_STATE_CD,
                put(compress(pty.party_id_country_code,'$'),$_ctry_cd_3to2_fmt.) as X_ID_AUTHORITY_COUNTRY_CD,  
                compress(pty.email_address,'$') as X_EMAIL_ADDRESS_TXT,
                
                ifc(pty.party_type_desc='INDIVIDUAL','C','') as X_GENDER_CD  

                    

%mend;

 

%macro ctr_extract_branch_data(inDsn=,outDsn=);


  
  /*--------------------------------------------------------
   * Extract associated branches
   *-------------------------------------------------------*/

   
    %let branch_list=;
    proc sql noprint;
         select distinct strip(put(branch_key,best.))
                into :branch_list separated by ' '
         from &inDsn;
    quit; 

     %fcf_rcset(&sqlrc);
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: CTR: Cannot create list of branch data.;
          %fcf_put_error;
           
     %end;
   
    %let branch_date_where=%str(today() between x_branch_open_dt and x_branch_close_dt);

    %put &branch_list;
    %put &branch_date_where;

    %if %sysfunc(exist(&outdsn)) %then 
    %do;
        proc datasets nolist nowarn lib=work;
             delete &outdsn;
        quit;
    %end;

     

    %ecm_generic_data_get(   udf_table_nm=x_branch
                                     , udf_nm=x_branch_ext_key
                                     , udf_value_table_nm=generic_data_udf_num_value
                                     , udf_value_list=&branch_list                                   
                                     , where_cl=%superq(branch_date_where)
                                     , outdsn=&outdsn
                                     
                         );
   
    
    %if not %sysfunc(exist(&outdsn)) %then 
              
        %fcf_rcset(8);
    
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: CTR: Cannot retrieve branch data.;
          %fcf_put_error;
         
     %end;
 %mend;
 
 %macro ctr_custom_branch(inTxnDsn=,inBranchDsn=);

  /* SQL query to map the branch data to CTR Institution Occured data */

  proc sql noprint;
            create table x_institution as 
            select distinct 
                txn.alert_id as RR_RK, /* never change this */
                br.generic_data_rk  as X_INSTITUTION_RK,  
                br.x_branch_id as X_INST_ID,  
                br.x_branch_nm as X_INST_LEGAL_NM,
                br.x_branch_address_txt as X_INST_ADDRESS_TXT,
                br.x_branch_city_nm as X_INST_CITY_NM,
                br.x_branch_state_cd as X_INST_STATE_CD,
                br.x_branch_country_cd as X_INST_COUNTRY_CD,
                br.x_branch_zip_cd as X_INST_POSTAL_CD,
                br.x_parent_fed_regulator_cd as X_INST_PRIM_FED_REG_CD,
                br.x_parent_tax_id as X_INST_TIN_NO,
                br.x_parent_type_cd as X_INST_TYPE_CTR_CD,
                case transaction_cdi_code
                when 'C' then sum(txn.currency_amount)  
                else . 
                end as X_CASH_IN_AMT,
                case transaction_cdi_code
                when 'D' then sum(txn.currency_amount)  
                else . 
                end as X_CASH_OUT_AMT
              
    from  &inTxnDsn      as txn,
          &inBranchDsn    as br
       
    where txn.branch_key=br.x_branch_ext_key 
    group by txn.branch_key 
    order by rr_rk;

    quit;

    %fcf_rcset(&sqlrc);
    
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
     %let _fcfmsg_=ERROR: CTR: Cannot create CTR branch data.;
          %fcf_put_error;
         
     %end;     

          
%mend;



%macro ctr_custom_term;

/* Macro to be executed after converting FCF data into CTR in ctr_process.sas */

%mend;


%mend;

