/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    cdd_sm_import_rules.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  Windows, Unix
| DESC:    reads the package and updates the ECM database
+------------------------------------------------------------------------------+
| PARAMETERS:
|   path:
|     path on the server where to save the files from the package, but also
|     used as:
|       if type=F path for the package file
|       if type=I path for the package file
|
|   url:
|     URL for package (including the package name) if package is C
|
|   packageType:
|     F - process package as a file on the disk
|     I - install, process package, no lock needed
|
|   name:  if type=F then name of the file, no extension (assumed to be .spk)
|          if type=I then name of the XML file including
|   rename:
|     Y or N. If Y change all PARTY_IDs in the imported rules and rule groups
|     Y may be used to create a copy of a rule (copy is not implemented yet).
|
|   username: user name to use as update user in ECM tables
|
|   authdomain:
|     authentication domain to access Content Server
|
|   defaultLanguage:
|     if the parameter value is not empty then rule and group localizable data
|     is updated to that language using REF_TABLE_VALUE and REF_TABLE_TRANS,
|     so those tables should be populated before the update happens.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_sm_import_rules
        (url=, path=, packageType=F, name=cdd_export.spk, rename=N,
         username=, authdomain=, defaultLanguage=);

   %let oracle_upd_nowhere_option=;
   %macro cdd_oracle_options();
      %if %upcase(&ecm_dbserver) = ORACLE %then
      %do;
        %let oracle_upd_nowhere_option=(or_upd_nowhere=no);
      %end;
   %mend;
   %cdd_oracle_options

  /* options spool mprint mlogic symbolgen mprintnest mlogicnest; */

  /* reset error codes to zero */
  %let trans_rc = 0;
  %let sqlrc = 0;
  %let sqlxrc = 0;
  %let sysrc = 0;
  %let cdd_sm_party_type=CDDSM;
  %let cdd_sm_rule_cat=CDDRULE;
  %let cdd_sm_rule_group_cat=CDDRG;
  %let db_engine=ODBC;                           /* Let default DB engine be ODBC */

  %let fcf_sa_dav_err_flg = 0;
  %let cdd_sm_import_error_flg = 0;
    /* Change db_engine if the database server is Oracle or DB2 */
    %if %upcase(&ecm_dbserver) = ORACLE %then
    %do;
         %let db_engine=&ecm_dbserver;
    %end;
    %else %if %upcase(&ecm_dbserver) = DB2 %then
    %do;
          %let db_engine=&ecm_dbserver;
    %end;
    %else %if %upcase(&ecm_dbserver) = POSTGRES %then
    %do;
          %let db_engine=&ecm_dbserver;
    %end;

  %let worklib=cddimp;
  libname cddimp "&path";

  %let fcf_debug_flg=N;
  %macro fcf_debug_setup();
        %if &fcf_debug_flg eq Y %then %do;
            options spool mprint mlogic symbolgen mprintnest mlogicnest;
            libname dbg "!FCFROOT/custom";
        %end;
  %mend fcf_debug_setup;
  %fcf_debug_setup();

  %macro fcf_debug_copy();
        %if &fcf_debug_flg eq Y %then %do;
            proc copy inlib=work outlib=dbg;
            run;
            proc copy inlib=cddimp outlib=dbg;
            run;
        %end;
  %mend fcf_debug_copy;

  %macro cdd_sm_abort(msg=, p1=, p2=, p3=, p4=);
       %let parm_list=;
       %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
       %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
       %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
       %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

       %let _fcfmsg_=ERROR: %sysfunc(sasmsg(sashelp.cddmsg, &msg, N &parm_list));
       %let trans_rc = 99;
       %fcf_put_error
       %if &packageType ne I %then %do;
         %cdd_clear_sync_lock()
     %end;
       %ABORT CANCEL &trans_rc;
  %mend cdd_sm_abort;

  %macro cdd_sm_abort_on_error(msg=, p1=, p2=, p3=, p4=);
     %fcf_chkrc
     %if &bat_abort=Y %then %do;
         %let sys_err_msg = &bat_errmsg;
         %let parm_list=;
         %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
         %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
         %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
         %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

         %let _fcfmsg_=ERROR: Update failed: %sysfunc(sasmsg(sashelp.cddmsg, &msg, N &parm_list));
         %let trans_rc = 99;
         %fcf_put_error
         %if &packageType ne I %then %do;
             %cdd_clear_sync_lock();
         %end;
         %ABORT CANCEL &trans_rc;
     %end;
  %mend cdd_sm_abort_on_error;

  %macro cdd_upload_file_to_dav_server(davFileName=, localFileName=);
      filename scrsrv2 WEBDAV "&davFileName" AUTHDOMAIN="_SASMETA_";
      %cdd_sm_abort_on_error(msg=cdd_sm_error_uploading_sas_code, p1=&davFileName, p2=&localFileName)
      data _null_;
         infile "&localFileName";
         file scrsrv2;
         input;
         put _infile_;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_error_uploading_sas_code, p1=&davFileName, p2=&localFileName)
  %mend cdd_upload_file_to_dav_server;

  %macro cdd_sm_upload_files(dav_server_root=, dav_server_path=, category=, ds=);
      data _null_;
        set &ds;
        length locFileName davFileName $400
               new_file_nm $40;

        if PARTY_CATEGORY_CD eq "&category" then do;
            locFileName = "&path./" || trim(left(FILE_NM));

            if FILE_TYPE_CD = "HEADER" then do;
                NEW_FILE_NM = 'h' || trim(left(PARTY_RK_NEW)) || '.sas';
            end;
            if FILE_TYPE_CD = "FOOTER" then do;
                NEW_FILE_NM = 'f' || trim(left(PARTY_RK_NEW)) || '.sas';
            end;
            if FILE_TYPE_CD = "RULE" then do;
                NEW_FILE_NM = 'r' || trim(left(PARTY_RK_NEW)) || '.sas';
            end;
            davFileName = "&dav_server_root.&dav_server_path." || trim(left(NEW_FILE_NM));
            call execute ('%nrstr(%cdd_upload_file_to_dav_server(davFileName=' || davFileName ||', localFileName=' || locFileName || '))');
        end;
      run;
  %mend cdd_sm_upload_files;

  %ecm_db_connect

  /* lock on the dataset to make sure that only one sync job is running */
  %if &packageType ne I %then %do;
        %cdd_sync_lock()
        %if &cdd_sync_lock_failed eq Y %then %do;
             %cdd_sm_abort(msg=cdd_critical_update_already_running, p1=&cdd_sync_job_ds)
        %end;
  %end;

  /* create data set to store errors */
  data errors;
          length msg $400;
          stop;
  run;

  /* create events work table  */
  proc sql;
    create table cdd_event like ecm_db.ecm_event;
  quit;

  /* some of the code above may set SYSRC to a negative value, so reset */
  %let sysrc = 0;

  %if &packageType = F or &packageType = I %then %do;
      data errors;
         length properties nameValue entryType userSpec filename filepath channel $ 255;
         length abstract desc msg $2048;
         channel       = '';
         entryCount    = 0;
         entryId       = 0;
         packageId     = 0;
         packageListId = 0;
         rc            = 0;
         totalPackages = 0;
         packageId     = 0;
         numEntries    = 0;
         dateTime      = 0;

         retrievalType = 'FROM_ARCHIVE ';
         retrievalInfo = "&path./&name";
         properties    = '';

         CALL RETRIEVE_PACKAGE(packageListId, retrievalType, retrievalInfo, totalpackages, rc);
         if rc ne 0 then do;
            msg = sysmsg();
            put msg;
            output;
            call symput('cdd_sm_import_error_flg', '1');
         end;
       else do;
           CALL PACKAGE_FIRST(packageListId, packageId, numEntries, desc, dateTime, nameValue, channel, rc);
           if rc ne 0 then do;
              msg = sysmsg();
              put msg;
              output;
              call symput('cdd_sm_import_error_flg', '1');
           end;
           else do;
              put "packageId: "     packageId;
              put "  numEntries: "  numEntries;
              put "  Description: " desc;
              put "  nameValue: "   nameValue;
              put "  dateTime: "    dateTime datetime21.2 " GMT";
              put "  abstract: "    abstract;

             properties = 'FILENAME';
             do while (rc = 0);
                CALL ENTRY_NEXT(packageId, entryId, entryType, userSpec, desc, nameValue, rc,
                                properties, filename);
                if (rc = 0) then do;
                   entryCount = entryCount + 1;
                   put 'Entry: ' entryCount;
                   put '        EntryId: '     entryId;
                   put '        Type: '        entryType;
                   put '        UserSpec: '    userSpec;
                   put '        Description: ' desc;
                   put '        nameValue: '   nameValue;

                   if (entryType = 'TEXT' OR entryType = 'BINARY') then do;
                      filepath = "FILENAME: &path/" || filename;
                      CALL RETRIEVE_FILE(entryId, filepath, rc2);
                      if (rc2 ne 0) then do;
                         msg = sysmsg();
                         put msg;
                     output;
                   call symput('cdd_sm_import_error_flg', '1');
                      end;
                      else do;
                         put 'File retrieved: ' filepath;
                      end;
                   end;
                end;
                else do;
                   /* warning that there are no more files */
                   msg = sysmsg();
                   put msg;
                end;
             end; /* do while */

             CALL PACKAGE_TERM(packageId, rc);
             if rc ne 0 then do;
                msg = sysmsg();
                put msg;
                output;
                call symput('cdd_sm_import_error_flg', '1');
             end;
           end;
       end;
      run;
      %if &cdd_sm_import_error_flg ne 0 %then %do;
              %let cdd_sm_import_err=%sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_import_err, N));
              %put ERROR: &cdd_sm_import_err;
              %cdd_sm_abort(msg=cdd_sm_import_err);
          %goto exit;
      %end;
  %end;

  %if %sysfunc(fileexist("&path./cdd_export.xml")) ne 1 %then %do;
      %let cdd_sm_corrupted_package_err=%sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_corrupted_package_err, N, cdd_export.xml));
      data errors1;
              length msg $400;
              msg = "&cdd_sm_corrupted_package_err";
              output;
      run;
      proc append base=errors data=errors1 force;
      run;
      %let cdd_sm_import_error_flg = 1;
      %goto exit;
  %end;
  libname xmlin xml "&path./cdd_export.xml";

  proc copy in=xmlin out=cddimp memtype=data ;
  run;
  %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=&path./cdd_export.xml);

  /* check that the package has at least one rule */
  %let rule_count=0;
  proc sql noprint;
    select count(*) into :rule_count from &worklib..cdd_party_live
    where PARTY_TYPE_CD="&cdd_sm_party_type" and PARTY_CATEGORY_CD="&cdd_sm_rule_cat"
    ;
  quit;

  %if &rule_count = 0 %then %do;
      %let cdd_sm_no_rules_in_the_package_err=%sysfunc(sasmsg(sashelp.cddmsg, cdd_sm_no_rules_in_the_package_err, N));
      data errors1;
              length msg $400;
              msg = "&cdd_sm_no_rules_in_the_package_err";
              output;
      run;
      proc append base=errors data=errors1 force;
      run;
      %let cdd_sm_import_error_flg = 1;
      %goto exit;
  %end;

  /* fix datatypes, XML engine sets default types that do not match party_live */
  data &worklib..cdd_party_live;
         set &worklib..cdd_party_live (rename=(VALID_TO_DTTM=VALID_TO_DTTM_OLD INDIVIDUAL_FLG=INDIVIDUAL_FLG_OLD DELETE_FLG=DELETE_FLG_OLD IDENTICAL_PARTY_LINK_SK=IDENTICAL_PARTY_LINK_SK_OLD));
         drop VALID_TO_DTTM_OLD INDIVIDUAL_FLG_OLD DELETE_FLG_OLD IDENTICAL_PARTY_LINK_SK_OLD;

         length IDENTICAL_PARTY_LINK_SK VALID_TO_DTTM 8;

         if missing(IDENTICAL_PARTY_LINK_SK_OLD) then
             IDENTICAL_PARTY_LINK_SK = .;
         else
             IDENTICAL_PARTY_LINK_SK = input(IDENTICAL_PARTY_LINK_SK_OLD, BEST32. );

         VALID_TO_DTTM = .;

         if INDIVIDUAL_FLG_OLD = 1 then
            INDIVIDUAL_FLG = '1';
         else
            INDIVIDUAL_FLG = '0';

         if DELETE_FLG_OLD = 1 then
            DELETE_FLG = '1';
         else
            DELETE_FLG = '0';
    run;

  %if &rename eq Y %then %do;
      /* TODO: rename rules - will be used for Copy. Rename requires updatig ref tables before replacing them in ECM */
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=Copy);
  %end;

  data _null_;
     call symput('current_dt',put(datetime(),datetime22.));
  run;


/* ================================================================================= */
/* process ref_table_value and ref_table_trans                                       */
/* ================================================================================= */
    /* during install import ref tables are populated from SMD file, for regular import they should be
    *  populated before UDF fields are updated, so that default language may be corrected
    *  if needed - not done currently because source language is not passed in the package
    *  cdd_export.spk is shipped without ref tables - so this section will be executed
    *  but nothing will be updated */
    %if &packageType ne I %then %do;
        /* all values are updated */
        /* delete TRANS data first, need to delete records that are updated in ref_table_trans and records that have  parents in ref_table_value */
        %if %sysfunc(exist(cddimp.cdd_ref_table_trans)) or %sysfunc(exist(cddimp.cdd_ref_table_value)) %then %do;
            /* first delete CDD_RULE_PARM_DESCRIPTION, delete records that have parent in ref_table_value or need to be updated
             * VALUE_CD is unique, REF_TABLE_VALUE.PARENT_VALUE_CD is ID of the rule that owns the parameter
            */
            %if %sysfunc(exist(cddimp.cdd_ref_table_value)) %then %do;
                proc sql;
                  delete from ecm_db.ref_table_trans &oracle_upd_nowhere_option
                  where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION"
                      and VALUE_CD in (select VALUE_CD from cddimp.CDD_REF_TABLE_VALUE where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION")
                  ;
                quit;
            %end;
            %if %sysfunc(exist(cddimp.cdd_ref_table_trans)) %then %do;
                proc sql;
                  delete from ecm_db.ref_table_trans &oracle_upd_nowhere_option
                  where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION"
                      and VALUE_CD in (select VALUE_CD from cddimp.CDD_REF_TABLE_TRANS where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION")
                  ;
                quit;
            %end;
            proc sql;
              delete from ecm_db.ref_table_trans &oracle_upd_nowhere_option
              where
                  REF_TABLE_NM in ("CDD_RULE_NAME", "CDD_RULE_DESCRIPTION",
                             "CDD_RULE_GROUP_NAME", "CDD_RULE_GROUP_DESC")
                  and VALUE_CD in (select PARTY_ID from cddimp.cdd_party_live)
              ;
            quit;
        %end;

        /* then delete REF_TABLE_VALUE data and add new values */
        %if %sysfunc(exist(cddimp.cdd_ref_table_value)) %then %do;
            /* first delete CDD_RULE_PARM_DESCRIPTION - it has references to CDD_RULE_NAME table */
            proc sql;
              delete from ecm_db.ref_table_value
              where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION"
                    and PARENT_VALUE_CD in (select PARTY_ID from cddimp.cdd_party_live)
              ;
            quit;
            proc sql;
              delete from ecm_db.ref_table_value &oracle_upd_nowhere_option
              where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION"
                  and VALUE_CD in (select VALUE_CD from cddimp.CDD_REF_TABLE_VALUE where REF_TABLE_NM =  "CDD_RULE_PARM_DESCRIPTION")
              ;
            quit;
            proc sql;
              delete from ecm_db.ref_table_value
              where REF_TABLE_NM in ("CDD_RULE_NAME", "CDD_RULE_DESCRIPTION",
                    "CDD_RULE_GROUP_NAME", "CDD_RULE_GROUP_DESC")
                    and VALUE_CD in (select PARTY_ID from cddimp.cdd_party_live)
              ;
            quit;
            /* fix datatypes, XML engine sets default types that do not match ref_table_value */
            /* ACTIVE_FLG is CHAR(1) in ECM, it is a number after XML import */
            data cddimp.cdd_ref_table_value;
                   set cddimp.cdd_ref_table_value (rename=(ACTIVE_FLG=ACTIVE_FLG_OLD));
                   drop ACTIVE_FLG_OLD;
                   length ACTIVE_FLG $1;
                   if missing(ACTIVE_FLG_OLD) then ACTIVE_FLG = ' ';
                   if ACTIVE_FLG_OLD = 1      then ACTIVE_FLG = '1';
                   else                            ACTIVE_FLG = '0';
            run;

            /* now add new VALUES data */
            /* if PARTY_ID has been changed for rename then it must be also changed in the ref tables */
            /* append must add CDD_RULE_PARM_DESCRIPTION last because REF_TABLE_VALUE has FK Constraints
             * and the order of rows is important for some DMBS, at least for Db2 we must insert parent records first */
            proc sql;
                create table CDD_REF_TABLE_VALUE_1 as
                  select * from cddimp.CDD_REF_TABLE_VALUE
                  where REF_TABLE_NM ne  "CDD_RULE_PARM_DESCRIPTION"
                  ;
            quit;
            %if &SQLOBS ne 0 %then %do;
                proc append base=ecm_db.ref_table_value data=CDD_REF_TABLE_VALUE_1 FORCE;
                run;
                %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.ref_table_value_1);
            %end;
            proc sql;
                create table CDD_REF_TABLE_VALUE_2 as
                  select * from cddimp.CDD_REF_TABLE_VALUE
                  where REF_TABLE_NM eq  "CDD_RULE_PARM_DESCRIPTION"
                  ;
            quit;
            %if &SQLOBS ne 0 %then %do;
                proc append base=ecm_db.ref_table_value data=CDD_REF_TABLE_VALUE_2 FORCE;
                run;
                %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.ref_table_value_2);
            %end;
        %end;
        /* then add new TRANS data */
        %if %sysfunc(exist(cddimp.cdd_ref_table_trans)) %then %do;
            /* TODO: when and if PARTY_ID is changed for rename then it must be also changed in the ref tables */
            /* rename is not implemented yet */
            proc append base=ecm_db.ref_table_trans data=cddimp.CDD_REF_TABLE_TRANS FORCE;
            run;
            %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.ref_table_trans);
        %end;
    %end;

    /* There are two cases: install and regular import
     * For install
     *      ref_table_value contains default language for this system = &defaultLanguage
     *      ref_table_trans contains all other languages except default language
     *      cdd_export.spk entities language is English
     *    Need to update entities with default language if &defaultLanguage != en, default lang will be in ref_table_value
     * For regular import
     *      ref_table_value contains default language for the source which could be not the same as this (target) system, i.e. not &defaultLanguage
     *      ref_table_trans contains all other languages except default language for the source system
     *      package contains entities where language is the source system default language
     *   TODO: pass the language in the export file, when this is done:
     *      During promotion if the target default language is not the same as the source then we need to update
     *      the entities in the package with values in the target lang, and above we need to correct ref_table_value to contaion that language
     *      For now, do not anything, assume language is the same
     */

    /* using the fact that  cdd_ref_table_value will not exists only in cdd-export.spk during install */
    %if &defaultLanguage ne  and &defaultLanguage ne en and not %sysfunc(exist(cddimp.cdd_ref_table_value)) %then %do;
        /* TODO: modify localizable data using specified language */
        /* first see if locale exists in ref_table_trans */
        /*
        /* ref_table_value contains the target system default language */
        proc sql;
            create table ref_tables1 as
                select REF_TABLE_NM, VALUE_CD, VALUE_DESC, PARENT_VALUE_CD
                from ecm_db.ref_table_value
                where REF_TABLE_NM in ("CDD_RULE_NAME", "CDD_RULE_DESCRIPTION",
                         "CDD_RULE_GROUP_NAME", "CDD_RULE_GROUP_DESC", "CDD_RULE_PARM_DESCRIPTION")
            ;
        quit;
        proc sql;
                create table ref_tables as
                    select r.REF_TABLE_NM, r.VALUE_CD, r.VALUE_DESC, p.PARTY_RK, c.ROW_NO
                    from ref_tables1 r
                    left join cddimp.cdd_party_live p
                    on r.VALUE_CD = p.PARTY_ID or r.PARENT_VALUE_CD = p.PARTY_ID
                    left join  cddimp.cdd_party_udf_char c
                    on  c.PARTY_RK = p.PARTY_RK and c.UDF_NM = "CDD_RULE_PARM_DESC_ID" and c.UDF_VALUE = r.VALUE_CD
                 ;
        quit;
        /* now update PARTY_FULL_NM for rules and groups */
        proc sql;
            update cddimp.cdd_party_live p
                set PARTY_FULL_NM = COALESCE(
                                             (select VALUE_DESC from ref_tables where REF_TABLE_NM = "CDD_RULE_NAME" and VALUE_CD=p.PARTY_ID),
                                             PARTY_FULL_NM
                                            )
                where p.PARTY_CATEGORY_CD = "&cdd_sm_rule_cat"
            ;
            update cddimp.cdd_party_live p
                set PARTY_FULL_NM = COALESCE(
                                             (select VALUE_DESC from ref_tables where REF_TABLE_NM = "CDD_RULE_GROUP_NAME" and VALUE_CD=p.PARTY_ID),
                                             PARTY_FULL_NM
                                            )
                where p.PARTY_CATEGORY_CD = "&cdd_sm_rule_group_cat"
            ;
        quit;
        /* update UDF fields for name and description for rules and groups */
        proc sql;
            update cddimp.cdd_party_udf_char p
                set UDF_VALUE = COALESCE(
                                         (select VALUE_DESC from ref_tables r where REF_TABLE_NM = p.UDF_NM and r.PARTY_RK=p.PARTY_RK),
                                         UDF_VALUE
                                        )
                where UDF_TABLE_NM="PARTY" and UDF_NM in ("CDD_RULE_NAME", "CDD_RULE_DESCRIPTION", "CDD_RULE_GROUP_NAME", "CDD_RULE_GROUP_DESC")
            ;
        quit;
        /* update UDF fields for parameter description */
        proc sql;
            update cddimp.cdd_party_udf_char p
                set UDF_VALUE = COALESCE(
                                         (select VALUE_DESC from ref_tables r where REF_TABLE_NM = p.UDF_NM and r.PARTY_RK=p.PARTY_RK and p.ROW_NO=r.ROW_NO),
                                         UDF_VALUE
                                        )
                where UDF_TABLE_NM="X_CDD_RULE_PARAM" and UDF_NM="CDD_RULE_PARM_DESCRIPTION"
            ;
        quit;
    %end;

    /* select all CDD entities from PARTY_LIVE to improve performance of joins later, this table will be quite small */
      proc sql;
        create table &worklib..party_live as
            select * from ecm_db.party_live p
            where   "&cdd_source_system_cd" = p.SOURCE_SYSTEM_CD and PARTY_TYPE_CD = "&cdd_sm_party_type"
            ;
      quit;

    /* now split existing and new parties */
      proc sql;
        create table &worklib..p_live_new as
        select * from cddimp.cdd_party_live p
            where  (select count(*) from &worklib..party_live e where p.party_id = e.party_id) = 0
        ;
      quit;
      %let new_count = &SQLOBS;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=p_live_new);

      /* list of existing parties, party_rk from the target system should be preserved, version_no needs to be incremented */
      proc sql;
        create table &worklib..p_live_upd as
        select p.*, e.party_rk as party_rk_new, (e.version_no +1 ) as version_no_new
          from cddimp.cdd_party_live p
          left join &worklib..party_live e
          on p.party_id = e.party_id and "&cdd_source_system_cd" = e.SOURCE_SYSTEM_CD
        where  not (p.party_rk in select party_rk from &worklib..p_live_new)
        ;
      quit;
      %let upd_count = &SQLOBS;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=p_live_upd);


/* ================================================================================= */
/* add new rules and groups - PARTY_RK has to be changed to use target system
 * sequence to avoid duplicates in the future */
/* ================================================================================= */
  %if &new_count > 0 %then %do;
      data &worklib..p_live_new;
           length source_system_cd $10; /* need to override xml file length, it could be shorter than database */
           length update_user_id $60;
           set &worklib..p_live_new (rename=(party_rk=party_rk_old));

            source_system_cd = "&cdd_source_system_cd";
            version_no = 1;
            update_user_id = "&username";
            valid_from_dttm="&current_dt"dt;
            create_dttm="&current_dt"dt;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=p_live_new_rename);

      %fcf_get_ecm_sequence_ids(in=&worklib..p_live_new,out=&worklib..p_live_new,type=party,block=no);

      proc append base=ecm_db.party_live data=&worklib..p_live_new force;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.party_live);
      proc append base=ecm_db.party_version data=&worklib..p_live_new force;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.party_version);

        /* add LOADEN events to CDD_EVENT */
      proc sql;
        create table &worklib..evt_new as
           select
              . as EVENT_RK                                              length = 8 format = 11. informat = 11.,
              ("PARTY") as BUSINESS_OBJECT_NM        length = 100  format = $100. informat = $100.,
              PARTY_RK as BUSINESS_OBJECT_RK,
              ("LOADEN") as EVENT_TYPE_CD                    length = 10 format = $10. informat = $10.,
              ("fcf.event.import.txt") as EVENT_DESC   length = 256 format = $256. informat = $256.,
              CREATE_USER_ID,
              CREATE_DTTM,
              "" as LINKED_OBJECT_NM                             length = 100 format = $100. informat = $100.,
              . as LINKED_OBJECT_RK                              length = 8 format = 11. informat = 11.
        from &worklib..p_live_new
        ;
      quit;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=evt_new);

      proc append base=cdd_event data=&worklib..evt_new force;
      run;
      %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=evt_new_append);

        /* add UDF char fields */
        %if %sysfunc(exist(cddimp.cdd_party_udf_char)) %then %do;
          proc sql;
            create table &worklib..p_udfc_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_char p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;

          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfc_new;
                     set &worklib..p_udfc_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                    valid_from_dttm="&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_CHAR_VALUE data=&worklib..p_udfc_new force;
            run;
            %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udf_char_value);
            %end;
        %end;

        %if %sysfunc(exist(cddimp.cdd_party_udf_date)) %then %do;
              /* add UDF date fields */
          proc sql;
            create table &worklib..p_udfd_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_date p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;

          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfd_new;
                     set &worklib..p_udfd_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                    valid_from_dttm="&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_DATE_VALUE data=&worklib..p_udfd_new force;
            run;
                    %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udf_date_value);
            %end;
        %end;

        %if %sysfunc(exist(cddimp.cdd_party_udf_num)) %then %do;
              /* add UDF num fields */
          proc sql;
            create table &worklib..p_udfn_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_num p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;

          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfn_new;
                     set &worklib..p_udfn_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                     valid_from_dttm="&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_NUM_VALUE data=&worklib..p_udfn_new force;
              run;
              %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udf_num_value);
            %end;
        %end;

        %if %sysfunc(exist(cddimp.cdd_party_udf_lgchr)) %then %do;
              /* add UDF large char fields */
          proc sql;
            create table &worklib..p_udfl_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_lgchr p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfl_new;
                     set &worklib..p_udfl_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                     valid_from_dttm="&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_LGCHR_VALUE data=&worklib..p_udfl_new force;
              run;
              %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udf_lgchr_value);
            %end;
        %end;

        /* update access in PARTY_X_USER_GROUP  */
        proc sql;
          create table &worklib..cdd_groups as
          select c.PARTY_TYPE_CD, c.PARTY_CATEGORY_CD, u.USER_GROUP_NM
          from ecm_db.party_config c
          join ecm_db.PARTY_CONFIG_X_USER_GROUP u
          on c.PARTY_CONFIG_SEQ_NO = u.PARTY_CONFIG_SEQ_NO
             and c.PARTY_TYPE_CD = "&cdd_sm_party_type"
             and (c.PARTY_CATEGORY_CD = "&cdd_sm_rule_cat" or c.PARTY_CATEGORY_CD = "&cdd_sm_rule_group_cat")
                ;
        quit;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=cdd_groups);

        proc sql;
          insert into ecm_db.PARTY_X_USER_GROUP (PARTY_RK, USER_GROUP_NM)
            select p.PARTY_RK, g.USER_GROUP_NM
             from &worklib..p_live_new p
             join &worklib..cdd_groups g
             on p.PARTY_TYPE_CD = g.PARTY_TYPE_CD and p.PARTY_CATEGORY_CD = g.PARTY_CATEGORY_CD
             ;
        quit;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.PARTY_X_USER_GROUP);

        /* upload files */
        proc sql;
          create table &worklib..cdd_files_new as
          select f.*, p.party_rk as party_rk_new from &worklib..cdd_files f
          join &worklib..p_live_new p
          on f.party_rk = p.party_rk_old
          ;
        run;

        %cdd_sm_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&cdd_rules_dav_path,       category=CDDRULE, ds=&worklib..cdd_files_new)
        %cdd_sm_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&cdd_rule_groups_dav_path, category=CDDRG,   ds=&worklib..cdd_files_new)
       /* party_x_party is processed after updated parties are processed so that we have all references */
  %end;

/* ================================================================================= */
/* process exsisting rules and groups - ECM existing PARTY_RK does not change in ECM */
/* ================================================================================= */
  %if &upd_count > 0 %then %do;
      /* use existing PARTY_RK */
        data &worklib..p_live_upd;
             set &worklib..p_live_upd (rename=(party_rk=party_rk_old party_rk_new=party_rk version_no=version_no_old version_no_new=version_no));
             drop version_no_old;
  	         VALID_FROM_DTTM = "&current_dt"dt;
             VALID_TO_DTTM = .;
             UPDATE_USER_ID = "&username";
        run;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=p_live_upd_rename);

	    proc datasets lib = ecm_db nolist nowarn memtype = (data view);
	       delete tmp_party_staging;
	    quit;
        proc sql;
	        create table ecm_db.tmp_party_staging like ecm_db.party_live;
        quit;
	    proc append base=ecm_db.tmp_party_staging data=&worklib..p_live_upd FORCE;
	    run;

        /* update existing records with new info, not all fields need to be updated */
	    /* update CDD parties in party_live */
	    proc sql feedback;
	        connect to &db_engine as &db_engine (authdomain=&ecm_dbauth &ecm_db_conopts.);
	        execute (
	            merge into party_live pl
	            using (select party_rk, version_no, valid_from_dttm, party_full_nm, update_user_id from tmp_party_staging) tmp_party
	            on (tmp_party.party_rk = pl.party_rk)
	            when matched then update set
	                pl.VALID_FROM_DTTM=tmp_party.VALID_FROM_DTTM,
	                pl.PARTY_FULL_NM=tmp_party.PARTY_FULL_NM,
                    pl.UPDATE_USER_ID = tmp_party.UPDATE_USER_ID,
	                pl.VERSION_NO = tmp_party.VERSION_NO
                    %if %upcase(&db_engine) eq ODBC %then %do;
                        ;
                    %end;
	        ) by &db_engine;
	        disconnect from &db_engine;
	    quit;

        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.party_live.update);
        /* mark existing version as expired */
	    /* close party_version records for updated parties
	     * cannot use pv.valid_to_dttm is null in the ON clause because Oracle does not allow
	     * update columns that are used in the ON clause.
	     * Instead using fact that pv.valid_to_dttm is null for the same records where
	     * pv.version_no = tmp_party.version_no - 1
	    */
	    proc sql feedback;
	        connect to &db_engine as &db_engine (authdomain=&ecm_dbauth &ecm_db_conopts.);
	        execute (
	            merge into party_version pv
	            using (select party_rk, valid_from_dttm, version_no from tmp_party_staging) tmp_party
	            on (tmp_party.party_rk = pv.party_rk and  pv.version_no = tmp_party.version_no - 1)
	            when matched then update set
	                pv.VALID_TO_DTTM = tmp_party.VALID_FROM_DTTM
                    %if %upcase(&db_engine) eq ODBC %then %do;
            ;
                    %end;
	            ) by &db_engine;
	        disconnect from &db_engine;
       quit;
       %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.party_version);
       /* add new version to party_version*/
	    proc append base=ecm_db.party_version data=&worklib..p_live_upd FORCE;
	    run;
       %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.party_version_insert);
        /* add SAVENT events to CDD_EVENT */
       proc sql;
        create table &worklib..evt_upd as
           select
              . as EVENT_RK                                 length = 8 format = 11. informat = 11.,
              ("PARTY") as BUSINESS_OBJECT_NM   length = 100 format = $100. informat = $100.,
              PARTY_RK as BUSINESS_OBJECT_RK,
              ("SAVENT") as EVENT_TYPE_CD               length = 10 format = $10. informat = $10.,
              ("fcf.event.import.version.fmt.txt" || "09"x || trim(left(put(VERSION_NO, BEST32.)))) as EVENT_DESC length = 256 format = $256. informat = $256.,
              "&username" as CREATE_USER_ID,
              "&current_dt"dt as CREATE_DTTM    length = 8 format = DATETIME27.6 informat = DATETIME27.6,
              "" as LINKED_OBJECT_NM                    length = 100 format = $100. informat = $100.,
              . as LINKED_OBJECT_RK                     length = 8format = 11. informat = 11.
        from &worklib..p_live_upd
        ;
       quit;
       %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=evt_upd);

       proc append base=cdd_event data=&worklib..evt_upd force;
       run;
       %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=evt_new_append);

        /* update UDF char fields */
       %if %sysfunc(exist(cddimp.cdd_party_udf_char)) %then %do;
          proc sql;
            create table &worklib..p_udfc_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_char p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
            data &worklib..p_udfc_upd;
                     set &worklib..p_udfc_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
          	         VALID_FROM_DTTM = "&current_dt"dt;
            run;
            proc append base=ecm_db.PARTY_UDF_CHAR_VALUE data=&worklib..p_udfc_upd force;
            run;
            %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udfc_upd);
         %end;
      %end;

        /* update UDF date fields */
        %if %sysfunc(exist(cddimp.cdd_party_udf_date)) %then %do;
          proc sql;
            create table &worklib..p_udfd_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_date p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfd_upd;
                     set &worklib..p_udfd_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
          	         VALID_FROM_DTTM = "&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_DATE_VALUE data=&worklib..p_udfd_upd force;
              run;
              %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udfd_upd);
          %end;
        %end;

        /* update UDF num fields */
        %if %sysfunc(exist(cddimp.cdd_party_udf_num)) %then %do;
          proc sql;
            create table &worklib..p_udfn_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_num p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfn_upd;
                     set &worklib..p_udfn_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
          	         VALID_FROM_DTTM = "&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_NUM_VALUE data=&worklib..p_udfn_upd force;
            run;
            %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udfc_upd);
          %end;
        %end;

            /* update UDF large char fields */
        %if %sysfunc(exist(cddimp.cdd_party_udf_lgchr)) %then %do;
          proc sql;
            create table &worklib..p_udfl_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from cddimp.cdd_party_udf_lgchr p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfl_upd;
                     set &worklib..p_udfl_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
          	         VALID_FROM_DTTM = "&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_LGCHR_VALUE data=&worklib..p_udfl_upd force;
            run;
            %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_udfl_upd);
          %end;
        %end;

        /* no need to update PARTY_X_USER_GROUP  - not changing access on the target system */

      /* upload files */
      proc sql;
        create table &worklib..cdd_files_upd as
        select f.*, p.party_rk as party_rk_new from &worklib..cdd_files f
        join &worklib..p_live_upd p
        on f.party_rk = p.party_rk_old
        ;
      quit;

      %cdd_sm_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&cdd_rules_dav_path,       category=CDDRULE, ds=&worklib..cdd_files_upd)
      %cdd_sm_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&cdd_rule_groups_dav_path, category=CDDRG,   ds=&worklib..cdd_files_upd)
  %end;

/* ================================================================================= */
/* process party_x_party                                                                 */
/* ================================================================================= */

    %if %sysfunc(exist(cddimp.cdd_party_x_party)) %then %do;
        /* merge new and old parties back but with correct PARTY_RK */
        %if &upd_count > 0 %then %do;
            data &worklib..party_live_full_new;
                 set &worklib..p_live_upd;
            run;
            %if &new_count > 0 %then %do;
              proc append base=&worklib..party_live_full_new data= &worklib..p_live_new force;
              run;
            %end;
        %end;
        %else %do;
            data &worklib..party_live_full_new;
                 set &worklib..p_live_new;
        %end;

	    proc datasets lib = ecm_db nolist nowarn memtype = (data view);
	       delete tmp_party_staging;
	    quit;
	    proc sql;
	        create table ecm_db.tmp_party_staging like ecm_db.party_live;
	    quit;
	    proc append base=ecm_db.tmp_party_staging data=&worklib..party_live_full_new FORCE;
	    run;

        /* find rules that are already in a group and delete that relationship */
        proc sql;
          create table &worklib..party_x_party_del as
             select * from ecm_db.party_x_party
             where member_party_rk in ( select party_rk from  ecm_db.tmp_party_staging)
          ;
        quit;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_x_party_del);

        proc sql;
          delete from ecm_db.party_x_party
             where member_party_rk in ( select party_rk from  ecm_db.tmp_party_staging)
          ;
        quit;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_x_party_del);

        /* create remove relationship events */
        proc sql;
            create table &worklib..evt_pxp_del as
               select
                  . as EVENT_RK                                                 length = 8 format = 11. informat = 11.,
                  ("PARTY") as BUSINESS_OBJECT_NM       length = 100 format = $100. informat = $100.,
                  PARTY_RK as BUSINESS_OBJECT_RK,
                  ("REMPAR") as EVENT_TYPE_CD                   length = 10 format = $10. informat = $10.,
                  ("rg_X_rule") as EVENT_DESC           length = 256 format = $256. informat = $256.,
                  "&username" as CREATE_USER_ID,
                  "&current_dt"dt as CREATE_DTTM                length= 8 format = DATETIME27.6 informat=DATETIME27.6,
                  "PARTY" as LINKED_OBJECT_NM           length = 100 format = $100. informat = $100.,
                  MEMBER_PARTY_RK as LINKED_OBJECT_RK length = 8 format = 11. informat = 11.
            from &worklib..party_x_party_del
            ;
        quit;
        proc append base=cdd_event data=&worklib..evt_pxp_del force;
        run;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=cdd_event.append.evt_pxp_del);

        /* create new relationships */
        proc sql;
          create table &worklib..p_x_p_new as
          select x.*, parent.party_rk as party_rk_new, child.party_rk as member_party_rk_new
            from cddimp.cdd_party_x_party x
            inner join &worklib..party_live_full_new parent
            on parent.party_rk_old = x.party_rk
            inner join &worklib..party_live_full_new child
            on child.party_rk_old = x.member_party_rk
          ;
        quit;
        %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=p_x_p_new);
        %if &SQLOBS > 0 %then %do;
            data &worklib..p_x_p_new;
                   set &worklib..p_x_p_new (rename=(party_rk=party_rk_old party_rk_new=party_rk member_party_rk=member_party_rk_old member_party_rk_new=member_party_rk));
                   drop party_rk_old member_party_rk_old;
            run;
        %end;

        %if %sysfunc(exist( &worklib..p_x_p_new )) %then %do;
          proc append base=ecm_db.party_x_party data=&worklib..p_x_p_new;
          run;
          %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=party_x_party.append.p_x_p_new);
            /* create event records for added relationships*/
          proc sql;
              create table &worklib..evt_pxp_new as
                 select
                    . as EVENT_RK                                               length = 8 format = 11. informat = 11.,
                    ("PARTY") as BUSINESS_OBJECT_NM     length = 100 format = $100. informat = $100.,
                    PARTY_RK as BUSINESS_OBJECT_RK,
                    ("ADDPAR") as EVENT_TYPE_CD                 length = 10 format = $10. informat = $10.,
                    ("rg_X_rule") as EVENT_DESC                 length = 256 format = $256. informat = $256.,
                    "&username" as CREATE_USER_ID,
                    "&current_dt"dt as CREATE_DTTM              length= 8 format = DATETIME27.6 informat=DATETIME27.6,
                    "PARTY" as LINKED_OBJECT_NM                 length = 100 format = $100. informat = $100.,
                    MEMBER_PARTY_RK as LINKED_OBJECT_RK length = 8 format = 11. informat = 11.
              from &worklib..p_x_p_new
              ;
          quit;
          %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=evt_pxp_new);
          proc append base=cdd_event data=&worklib..evt_pxp_new force;
          run;
          %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=cdd_event_pxp.append.evt_pxp_new);
        %end;
    %end;

  /* update ECM_EVENT table */
  %fcf_get_ecm_sequence_ids(in=cdd_event, out=cdd_event, type=event, block=no);
  %fcf_debug_copy();

  proc append base=ecm_db.ecm_event data=cdd_event;
  run;
  %cdd_sm_abort_on_error(msg=cdd_sm_processing_error, p1=ecm_db.ecm_event);

%exit:
%mend cdd_sm_import_rules;
