/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    ctr_validate.sas
| PRODUCT: SAS Currency Transaction Reporting
| SYSTEM:  Windows, Unix
| DESC:    validate CTR report data with FinCEN published rules.
|          
+------------------------------------------------------------------------------+
| USAGE:   %ctr_validate; (all draft CTR reports will be validated) 
| PARAMETERS:
|           update_workflowYN: Y for transitioning workflow when the report is clean
|           subset_type: RR, CASE, or INCIDENT. If subset type is CASE or INCIDENT, 
|                        RR_LIVE.PARENT_OBJECT_NM and PARENT_OBJECT_RK will be used 
|                        to filter the reports.                         
|           subset_dsn:  name of input dataset containing the keys of the records 
|                        to be validated
|                        If this parameter is empty, all draft CTR reports will be 
|                        validated. 
|           skip_pivotYN: Y for skipping the pivot tables. This is only useful for
|                         debugging when the temp tables are already created.
|           print_reportYN: Y produce a html output of the validation result
|           testYN:       Y for running all web services in test mode. 
|                         web services are used in workflow updates only.
|           stpYN:        Y for running the macro in stored process.
|                         For stored process, the output destination is _webout.
|
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/



%macro ctr_validate( update_workflowYN=Y
                    ,subset_type=RR
                    ,subset_dsn=                   
                    ,skip_pivotYN=N
                    ,print_reportYN=Y
                    ,testYN=N
                    ,stpYN=N 
                    );
    /*------------
     * Set Module
     *------------*/
    %let _fcfmodule_=CTR_VALIDATE;

    /*-----------------------------
     * Set start time for jobstats
     *-----------------------------*/
    %fcf_section_header(section=&_fcfmodule_, message_format=verbose);


    %let trans_rc=0;
    %let _fcfmsg_=;
    %let successYN=Y;


    %let skip_pivotYN=%upcase(&skip_pivotYN);
    %let print_reportYN=%upcase(&print_reportYN);
    %let update_workflowYN=%upcase(&update_workflowYN);
    %let subset_type=%upcase(&subset_type);
    %let stpYN=%upcase(&stpYN);
    %if &stpYN=Y %then 
    %do;
         %let update_workflowYN=N;
         %let skip_pivotYN=N;
    %end;

    %if &subset_type ne RR and &subset_type ne INCIDENT and &subset_type ne CASE %then 
    %do;
          %fcf_rcset(5);
          %let _fcfmsg_=ERROR: CTR: Parameter subset_type is not RR, INCIDENT, or CASE. ;
          %fcf_put_error;
          %goto MACRO_END;
    %end;

    %if &update_workflowYN=Y %then
    %do;

        /* Check whether ctr validate workflow path is defined or not. */
        /* If it is not, run ctr_global_mvar macro.                    */

        %if %symexist(ctr_validated_wf_path)=0 %then %ctr_global_mvar;

        /* If it does not exist in ctr_global-var, change the workflow update parameter to N */
        %if %symexist(ctr_validated_wf_path)=0 %then %let ctr_validated_wf_path=;

        %if %str(&ctr_validated_wf_path)=%str() %then 
        %do;
        %let _fcfmsg_=ERROR: CTR: Global CTR parameter CTR_VALIDATED_WF_PATH is not defined. Changing parameter update_workflowYN to N. ;
         %fcf_put_error;
         %let update_workflowYN=N;    
        %end;

    %end;

    %if %str(&subset_dsn)=%str() %then
    %do; /* If subset table is not provided, validate all ctrx with ready code equals to draft */

        %let rr_subset_dsn=rr_subset;
         data &rr_subset_dsn;
           set ecm_db.rr_live;
           where rr_ready_cd='D' and form_config_rk=212;
           keep rr_rk;
         run;
         %let _fcfmsg_=NOTE: CTR: Parameter subset_dsn is blank. Validate all CTR reports in 'Draft' stage. ;
         %fcf_put_error;
    %end;
    %else %if  &subset_type=RR %then 
          %let rr_subset_dsn=&subset_dsn;
    %else 
    %do; 
          /* if the subset type is CASE or INCIDENT */
          /* retreive the list of reports based on its parent keys */
          %let rr_subset_dsn=rr_subset;
          %let parent_list=;
        proc sql noprint;
               select distinct strip(put(&subset_type._rk,best.)) into :parent_list separated by ' '
               from &subset_dsn;

               create table &rr_subset_dsn as
               select rr_rk
               from ecm_db.rr_live
               where form_config_rk=212
                and rr_ready_cd='D'
                and parent_object_nm="&subset_type"
                and parent_object_rk in (&parent_list)
         ;
         quit;
    %end;

    /* if report subset table cannot be created or does not exist */
    %if not %sysfunc(exist(&rr_subset_dsn)) %then
    %do;
         %fcf_rcset(5);
         %let _fcfmsg_=ERROR: CTR: &rr_subset_dsn does not exist or cannot be created. ;
         %fcf_put_error;
         %goto MACRO_END;    
    %end; 

    /* If the pivot tables have not been created */
    %if &skip_pivotYN ne Y %then
    %do;
        /* extract CTRX data and transform it into rectangular structure */
        %ecmrr_pivot_fincen_data(rr_subset_dsn=&rr_subset_dsn, form_config_rk=212);
    %end;

    /* if ecmrr_pivot_fincen_data is not successful */
    %if &successYN=N %then 
    %do;
         %let _fcfmsg_=ERROR: CTR: Errors found when running ecmrr_pivot_fincen_data.;
         %fcf_put_error;
         %fcf_rcset(8);
         goto MACRO_END;
    %end;

    /* if rr_pivot contains no result. */
    %if %ecm_nobs(rr_pivot) = 0 %then %do;
         %let _fcfmsg_=Note: CTR: No valid CTR report data found.;
         %fcf_put_error;
         %fcf_rcset(3);
         %goto MACRO_END;
    %end;


    /* Load CTR validation code macros  */
    /* The macros will be called in SAS data steps, one for each input table. */
    filename ctrchk "&fcfroot/custom/source/ctrx_validation_code_macro.sas";

    %if %sysfunc(fexist(ctrchk))=0 %then
    %do;
        %let _fcfmsg_=ERROR: CTR: Validation code %sysfunc(pathname(ctrchk)) not found.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;
    %inc ctrchk;

    /* Create SAS formats for validating country code and state/province codes */

    proc sql;
         create table _loc_fmt as 
         (select distinct ref_table_nm as fmtname ,
                  value_cd as start , 
                  'VALID' as label , 
                  'C' as type 
          from ecm_db.ref_table_value
          where ref_table_nm='X_RT_COUNTRY'
         ) 
         union 
         (select distinct strip(ref_table_nm) as fmtname ,
             strip(parent_value_cd)||'_'||strip(value_cd) as start , 
             'VALID' as label , 
             'C' as type 
          from ecm_db.ref_table_value
          where ref_table_nm='X_RT_STATE_PROVINCE'
         )
         order by fmtname, start
    ;
    quit;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;

    proc format cntlin=_loc_fmt;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;



    %macro ctr_call_validate(table_nm=);

        /* Call the validate macro by CTR tables */
        /* Add a record to _T_ERROR for each error found. */


        %if %upcase(&table_nm)=RR_PIVOT %then %let source_table_nm=RR;
        %else %let source_table_nm=%upcase(&table_nm);
        data _t_error;
        if 0 then set ctrx_error;
        set &table_nm;
        keep error_cd rr_rk source_table_nm source_field_nm;
        retain source_table_nm "&source_table_nm";
          %CTRX_&table_nm;
        run;

        %fcf_rcset(&syserr);

        proc append base=ctrx_error data=_t_error force;
        run;



        /* remove the temp table for the next cycle */
        proc sql noprint;
        drop table _t_error;
        quit;


    %mend;


    /* Create a master table for errors from all CTR tables */

    data ctrx_error; 
         length source_table_nm $ 32
            source_field_nm $ 128
            rr_rk           8
            error_cd       $ 4
         ;
         stop;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;


    %ctr_call_validate(table_nm=RR_PIVOT);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
    %do;
        %let _fcfmsg_=ERROR: CTR: Errors found when validating RR_PIVOT.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;
    %ctr_call_validate(table_nm=R_X_PERSON_INVOLVED);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
    %do;
        %let _fcfmsg_=ERROR: CTR: Errors found when validating R_X_PERSON_INVOLVED.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;
    %ctr_call_validate(table_nm=R_X_INSTITUTION);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
    %do;
        %let _fcfmsg_=ERROR: CTR: Errors found when validating R_X_INSTITUTION.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;
    %ctr_call_validate(table_nm=R_X_FOREIGN_CASH_IN);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
    %do;
        %let _fcfmsg_=ERROR: CTR: Errors found when validating R_X_FOREIGN_CASH_IN.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;
    %ctr_call_validate(table_nm=R_X_FOREIGN_CASH_OUT);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
    %do;
        %let _fcfmsg_=ERROR: CTR: Errors found when validating R_X_FOREIGN_CASH_OUT.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;
    %ctr_call_validate(table_nm=R_X_TRAN_METHOD);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then 
    %do;
        %let _fcfmsg_=ERROR: CTR: Errors found when validating R_X_TRAN_METHOD.;
        %fcf_put_error;
        %fcf_rcset(99);
        %goto MACRO_END;
    %end;


    /* Check for missing branches and/or person involved */

    data _t_error;
         if 0 then set ctrx_error;
         merge rr_subset (keep=rr_rk in=in_master)
           r_x_institution (keep=rr_rk in=in_inst)
           R_x_person_involved (keep=rr_rk  in=in_person)
           rr_pivot (keep=rr_rk x_institution_rk
                 where=(x_institution_rk > 0 
                ) in=in_inst)
         ;
         by rr_rk;
         retain rr_rk error_cd;     
         source_table_nm='OVERALL';
         if not in_inst then
         do; 
         error_cd='F11';
         source_field_nm='RR.X_FILING_INST_*';
         output;
         end;
         if not in_inst then
         do; 

         error_cd='F12';
         source_field_nm='X_INSTITUTION.*';
         output;
         end;
         if not in_person then
         do; 
         error_cd='F14';
         source_field_nm='X_PERSON_INVOLVED.*';
         output;
         end;

         keep rr_rk source_table_nm source_field_nm error_cd;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;

    proc append base=ctrx_error data=_t_error force;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;

    proc sql noprint;
    drop table _t_error;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;

        %let clean_rr_cnt=0;    
        /* create a list of clean reports */
        proc sql;
        create table clean_rr_list as
        select distinct rr_rk                    
        from &rr_subset_dsn
        where rr_rk not in (select distinct rr_rk
                    from ctrx_error)
        ;    
        %let clean_rr_cnt=&sqlobs; 

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;


        %let error_rr_cnt=0;
        /* create a list of error reports */    
        proc sql;
        create table error_rr_list as   
        select distinct rr_rk 
        from ctrx_error
        ;    
        %let error_rr_cnt=&sqlobs;  

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %goto MACRO_END;
    %end;


    /**********************************************/
    /* Print Validation Result                    */
    /**********************************************/
    %if &print_reportYN=Y %then
    %do;
       %if &stpYN=Y %then 
       %do; 
           ods html body=_webout;
       %end; 
       %else
       %do; 
           %if "&printfile" ne "" %then
           %do;
           ods html body="&printfile" style=HTMLBlue;
           ods listing close;
        %end;
       %end;
        %if &clean_rr_cnt > 0 %then
        %do;

        %let clean_rr_list=;
        proc sql noprint; 
            select distinct strip(put(rr_rk,best.))                    
                    into :clean_rr_list separated by ' '
            from clean_rr_list;
            create table clean_rr_list_more  as
            select rr_rk, rr_id, rr_desc 
            from ecm_db.rr_live 
            where rr_rk in (&clean_rr_list);
        quit;

        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

         /* Produce clean reports */
         proc report data=clean_rr_list_more nowindows center ;
             column rr_id rr_desc;             
             define rr_id /width=10 flow "Report ID" ;
             define rr_desc /width=30 flow "Report Description";            
             title "CTR Validation Result - Clean Reports";      
         run;
         %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

        %end;


        %if &error_rr_cnt > 0 %then
        %do;

        %let error_rr_list=;
        proc sql noprint; 
            select distinct strip(put(rr_rk,best.))                    
                    into :error_rr_list separated by ' '
            from error_rr_list;
            create table error_rr_list_more  as
            select rr_rk, rr_id, rr_desc 
            from ecm_db.rr_live 
            where rr_rk in (&error_rr_list);
        quit;

        %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

        /**************************************/
        /* Obtain error description and types */
        /**************************************/


        proc sql;
        create table ctrx_error_more as 
        select a.rr_rk,  
           a.source_table_nm, a.source_field_nm, 
           b.*,
           c.rr_id, c.rr_desc
        from ctrx_error as a
        left join  ecm_form.ctrx_fincen_error_code as b
        on a.error_cd=b.error_cd
        join error_rr_list_more as c
        on a.rr_rk=c.rr_rk
        order by rr_id, error_cd
        ;
        quit;


        %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;

         proc format;
           value $errType
              'F'='Fatal'
              'P'='Primary'
              'S'='Secondary'
              ;
     run;

         /* Produce error reports */
         proc report data=ctrx_error_more nowindows center ;
             column rr_id rr_desc source_table_nm source_field_nm error_cd error_type_cd error_desc field_reference ;

             define rr_id /width=10 flow "Report ID" order;
             define rr_desc /width=30 flow "Report Description" order ;
             define source_table_nm /width=15 flow "Source Table Name";
             define source_field_nm /width=15 flow "Source Field Name";
          
            define error_desc /flow width=30;
            define error_type_cd /'Error Type' width=10 format=$errType.;                
             
        define field_reference /flow width=30 ; 

             title "CTRX Validation Result - Error Reports";      

          run;

           %fcf_rcset(&syserr);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
            %goto MACRO_END;
        %end;


          %end;

           %if &stpYN=Y or  "&printfile" ne "" %then
           %do;
          ods html close;
           %end;
    %end;


    %if &update_workflowYN=Y and &clean_rr_cnt > 0 %then
    %do;
        data ctr_wf_info;
         set clean_rr_list;
         retain wf_path &ctr_validated_wf_path
            wf_label &ctr_validated_wf_label
            wf_transition &ctr_validated_wf_transition

         ; 
         run;

          %fcf_rcset(&syserr);
          %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %goto MACRO_END;
          %end;


         %ecm_ws_wf_transition(datatype=RR,
                 indsn=ctr_wf_info,
                 outdsn=ctr_wf_info_ws,
                 path_var=wf_path,
                 label_var=wf_label,
                 applytransition_var=wf_transition
                 );

        %if &successYN=N %then 
        %do;
         %let _fcfmsg_=ERROR: CTR: Errors found when running ecm_ws_wf_transition.;
         %fcf_put_error;
         %fcf_rcset(8);
         goto MACRO_END;
        %end; 

        filename rwreq temp;
        %ecm_ws_writeXMl(indsn=ctr_wf_info_ws,outfref=rwreq,testmodeYN=&testYN);
        %if &successYN=N %then 
        %do;
         %let _fcfmsg_=ERROR: CTR: Errors found when running ecm_ws_wf_transition.;
         %fcf_put_error;
         %fcf_rcset(8);
         goto MACRO_END;
        %end; 

         filename rwresp temp;      

         %ecm_ws_soap(infref=rwreq, outfref=rwresp,cfgfref=soapcfg);

         %if &successYN=N %then 
         %do;
         %let _fcfmsg_=ERROR: CTR: Errors found when running ecm_ws_wf_transition.;
         %fcf_put_error;
         %fcf_rcset(8);
         goto MACRO_END;
        %end; 

          /* Embed the web service response to the log */     
        data _null_;
             retain failure_ind 0;
             infile rwresp;
             input;
             if index(_infile_,'resultCode="FAILURE"') then 
             do;
            call symput('_trans_rc',999);
            failure_ind=1;
             end;
             if failure_ind=1 then putlog _infile_;

        run;

        %if &trans_rc eq 999 %then 
        %do;
              %let _fcfmsg_=ERROR: CTR: Cannot update report workflow with ECM web service.;
              %fcf_put_error;
              %goto MACRO_END;
        %end;

    %end;
 

%MACRO_END:

     

    /* Set end time and calculate elapsed processing time for jobstats */
    %fcf_section_footer(section=&_fcfmodule_, message_format=verbose);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
    %else %do;
      %fcf_execute_macro(fcf_success);
    %end;
 
%mend ctr_validate;
