/*==============================================================================
| NAME:    fcf_load_dm_process
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  UNIX/WINDOWS
================================================================================

MODULE TYPE         - PROCESS
MODULE PURPOSE      - Extract, Aggregate, Transform and Load Customer Explorer data mart.

MODULE NOTES
------------
None

INPUTS
------
 None

PARAMETERS
------------
None

USAGE
-----
%fcf_ce_load_dm_process;

==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
%macro fcf_ce_load_dm_process;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/
        %let trans_rc=0;
        %let _fcfmsg_=;

    /* Clean-up from previous run */
    %put NOTE: FCF: Truncating Customer Explorer data mart tables;
    proc sql noprint;
      delete from seg_kc.fbu_customer_transaction;
      delete from seg_kc.fbu_alert_tx_agg;
      delete from seg_kc.fbu_customer_tx_agg;
      delete from seg_kc.fbu_customer_alert;
      delete from seg_kc.fbu_alerted_customer;
    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
      %let _fcfmsg_=ERROR: FCF: Cannot delete data from datamart.;
      %fcf_put_error;
      %goto MACRO_END;
    %end;
    /* Load customers with alerts */
    %put NOTE: FCF: Loading customers with alerts;
    %fcf_ce_load_alerted_customers;
    %fcf_get_observation_count(table=seg_kc.fbu_alerted_customer);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fbu_alerted_customer does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load customer alerts */
    %put NOTE: FCF: Loading customer alerts;
    %fcf_ce_load_customer_alerts;
    %fcf_get_observation_count(table=seg_kc.fbu_customer_alert);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fbu_customer_alert does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load transactions for customers with alerts */
    %put NOTE: FCF: Loading customer transactions;
    %fcf_ce_load_customer_tx;
    %fcf_get_observation_count(table=seg_kc.fbu_customer_transaction);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fbu_customer_transaction does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load transaction aggregates for customers */
    %put NOTE: FCF: Loading transaction aggregates for customers;
    %fcf_ce_load_customer_tx_agg;
    %fcf_get_observation_count(table=seg_kc.fbu_customer_tx_agg);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fbu_customer_tx_agg does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;

    /* Load transaction aggregates for customer alerts */
    %put NOTE: FCF: Loading transaction aggregates for customer alerts;
    %fcf_ce_load_alert_tx_agg;
    %fcf_get_observation_count(table=seg_kc.fbu_alert_tx_agg);
      %if &fcf_obs_count eq -1 %then %do;
         %let _fcfmsg_=NOTE: FCF: seg_kc.fbu_alert_tx_agg does not exist.;
         %fcf_put_error;
         %put &_fcfmsg_;
         %goto MACRO_END;
      %end;
      %else %if &fcf_obs_count eq 0 %then %do;
         %let _fcfmsg_=NOTE: FCF: No records to process.  Skipping execution.;
         %fcf_put_error;
         %goto MACRO_END;
      %end;
    /* Calculate and update transactions amount and count for customers with alerts */
    %put NOTE: FCF: Updating transactions amount and count for customers with alerts;
    proc sql noprint;
        create table customer_tx_aggregates as Select customer_key, count(*) As cnt,sum(transaction_amt) as amt
                                                 from seg_kc.fbu_customer_transaction
                                                 where triggering_indicator_cd='Y'
                                                 group by customer_key;
        update seg_kc.fbu_alerted_customer set aggregate_amt = 0;
        update seg_kc.fbu_alerted_customer set transactions_cnt = 0;
        update seg_kc.fbu_alerted_customer as A set aggregate_amt = (select amt from customer_tx_aggregates as B
               where A.customer_key = B.customer_key) where A.customer_key in (select customer_key from customer_tx_aggregates);

        update seg_kc.fbu_alerted_customer as A set transactions_cnt = (select cnt from customer_tx_aggregates as B
               where A.customer_key = B.customer_key) where A.customer_key in (select customer_key from customer_tx_aggregates);

    quit;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
       %let _fcfmsg_=ERROR: FCF: Cannot delete data from datamart.;
       %fcf_put_error;
       %goto MACRO_END;
    %end;

    /* Clean-up: Delete temporary customer_tx_aggregates dataset */
    %put NOTE: FCF: Deleting temporary customer_tx_aggregates dataset;
    proc datasets library=work nolist;
        delete customer_tx_aggregates;
    quit;

    %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %fcf_execute_macro(fcf_error);
   %else %do;
      %fcf_execute_macro(fcf_success);
   %end;
%mend fcf_ce_load_dm_process;
