/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_sa_import_scenarios.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    reads the package and updates the ECM database
+------------------------------------------------------------------------------+
| USAGE:   %fcf_sa_import_scenarios
|              (path, url, packageType, name, rename, username, authdomain)
+------------------------------------------------------------------------------+
| PARAMETERS:
|     path
|         path on the server where to save the files extracted from the package, but also
|         used as:
|             if type=F path for the package file
|             if type=I path for the package file
|
|     url: URL for package (including the package name) if package is C
|
|     packageType
|         F - process package as a file on the disk
|         I - install, process package, no lock needed
|             in install mode this macro also updates UDF fields for name, description, etc.
|             using fcfsa.smd file as the source and server config locale for language selection
|         U - duplicate mode, all datasets are already unpacked
|     name
|         if type=F then name of the file, no extension (assumed to be .spk)
|         if type=I then name of the XML file including extension
|         if type=U then name is not used
|
|     rename
|         Y or N. If Y change all PARTY_IDs in the imported scenarios and 
|         headers and routing groups.  
|         Y may be used to create a copy of a header (copy is not implemented yet).
|
|     username: user name to use as update user in ECM tables
|
|     authdomain: authentication domain to access Content Server
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_sa_update_udf_fields (src_type=, udf_nm=, udf_table_nm=);
        /* not using VALID_FROM_DTTM because at this time there is only one record for each entity */
        proc sql;
            update ecm_db.PARTY_UDF_CHAR_VALUE p
                set UDF_VALUE = COALESCE(
                                         (select UDF_VALUE from fcfsa_u r where type="&src_type" and r.PARTY_RK=p.PARTY_RK  and r.row_no = p.row_no),
                                         UDF_VALUE
                                        )
                where UDF_TABLE_NM="&udf_table_nm" and UDF_NM = "&udf_nm"
            ;
        quit;
%mend fcf_sa_update_udf_fields;

%macro fcf_sa_import_scenarios
           (url=, path=, packageType=F, name=fcf_export,
            rename=N, username=, authdomain=);
            
  /* reset error codes to zero */
  %let trans_rc = 0;
  %let sqlrc = 0;
  %let sqlxrc = 0;
  %let sysrc = 0;

  %let fcf_sa_dav_err_flg = 0;
  %let fcf_sa_import_error_flg = 0;

  %let worklib=fcfimp;
  %let work_path=&path;
  libname fcfimp "&work_path";

  %let fcf_sa_party_type=AMLSA;

    %let db_engine=ODBC;
    /* Change db_engine if the database server is Oracle or DB2 */
    %if %upcase(&ecm_dbserver) = ORACLE %then
    %do;
         %let db_engine=&ecm_dbserver;
    %end;
    %else %if %upcase(&ecm_dbserver) = DB2 %then
    %do;
          %let db_engine=&ecm_dbserver;
    %end;
    %else %if %upcase(&ecm_dbserver) = POSTGRES %then
    %do;
          %let db_engine=&ecm_dbserver;
    %end;

  %macro fcf_sa_abort(msg=, p1=, p2=, p3=, p4=);
       %let parm_list=;
       %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
       %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
       %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
       %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

       %let _fcfmsg_=ERROR: %sysfunc(sasmsg(sashelp.fcfmsg, &msg, N &parm_list));
       %let trans_rc = 99;
       %fcf_put_error
       %if &packageType ne I %then %do;
            %fcf_clear_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds)
       %end;
       %ABORT CANCEL &trans_rc;
  %mend fcf_sa_abort;

  %macro fcf_sa_abort_on_error(msg=, p1=, p2=, p3=, p4=);
     %fcf_chkrc
     %if &bat_abort=Y %then %do;
         %let sys_err_msg = &bat_errmsg;
         %let parm_list=;
         %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
         %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
         %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
         %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

         %let _fcfmsg_=ERROR: Update failed: %sysfunc(sasmsg(sashelp.fcfmsg, &msg, N &parm_list));
         %let trans_rc = 99;
         %fcf_put_error
         %if &packageType ne I %then %do;
             %fcf_clear_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds);
         %end;
         %ABORT CANCEL &trans_rc;
     %end;
  %mend fcf_sa_abort_on_error;

  %macro fcf_upload_file_to_dav_server(davFileName=, localFileName=);
      filename scrsrv2 WEBDAV "&davFileName" AUTHDOMAIN="_SASMETA_";
      %fcf_sa_abort_on_error(msg=fcf_sa_error_uploading_sas_code, p1=&davFileName, p2=&localFileName)
      %put Uploading  &localFileName to &davFileName;
      data _null_;
         infile "&localFileName";
         file scrsrv2;
         input;
         put _infile_;
      run;
      %fcf_sa_abort_on_error(msg=fcf_sa_error_uploading_sas_code, p1=&davFileName, p2=&localFileName)
      filename scrsrv2 clear;
  %mend fcf_upload_file_to_dav_server;

  %macro fcf_sa_upload_files(dav_server_root=, dav_server_path=, category=, ds=);
      data _null_;
        set &ds;
        length locFileName davFileName $400
               new_file_nm $40;

        if PARTY_CATEGORY_CD eq "&category" then do;
            locFileName = "&work_path./" || trim(left(FILE_NM));
            if NEW_FILE_NM ne " " then
                davFileName = "&dav_server_root.&dav_server_path." || trim(left(NEW_FILE_NM));
            else
                davFileName = "&dav_server_root.&dav_server_path." || trim(left(FILE_NM));

            if fileexist(locFileName) = 1 then do;
                call execute ('%nrstr(%fcf_upload_file_to_dav_server(davFileName=' || davFileName ||', localFileName=' || locFileName || '))');
            end;
        end;
      run;
  %mend fcf_sa_upload_files;

  %ecm_db_connect

  /* lock on the dataset to make sure that only one sync job is running */
  %if &packageType ne I %then %do;
        %fcf_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds)
        %if &fcf_sync_lock_failed eq Y %then %do;
             %fcf_sa_abort(msg=fcf_critical_update_in_process, p1=&fcf_sync_job_ds)
        %end;
  %end;

  /* create data set to store errors */
  data errors;
          length msg $400;
          stop;
  run;

  /* create events work table  */
  data &worklib..fcf_event;
      set ecm_db.ecm_event;
      stop;
  run;

  /* some of the code above may set SYSRC to a negative value, so reset */
  %let sysrc = 0;

  %if &packageType = F or &packageType = I %then %do;
      data errors;
         KEEP msg;
         length properties nameValue entryType userSpec filename filepath channel $ 255;
         length abstract desc msg $2048;
         channel       = '';
         entryCount    = 0;
         entryId       = 0;
         packageId     = 0;
         packageListId = 0;
         rc            = 0;
         totalPackages = 0;
         packageId     = 0;
         numEntries    = 0;
         dateTime      = 0;

         retrievalType = 'FROM_ARCHIVE ';
         retrievalInfo = "&path./&name";
         properties    = '';

         CALL RETRIEVE_PACKAGE(packageListId, retrievalType, retrievalInfo, totalpackages, rc);
         if rc ne 0 then do;
            msg = sysmsg();
            put msg;
            output;
            call symput('fcf_sa_import_error_flg', '1');
         end;
       else do;
           CALL PACKAGE_FIRST(packageListId, packageId, numEntries, desc, dateTime, nameValue, channel, rc);
           if rc ne 0 then do;
              msg = sysmsg();
              put msg;
              output;
              call symput('fcf_sa_import_error_flg', '1');
           end;
           else do;
              put "packageId: "     packageId;
              put "  numEntries: "  numEntries;
              put "  Description: " desc;
              put "  nameValue: "   nameValue;
              put "  dateTime: "    dateTime datetime21.2 " GMT";
              put "  abstract: "    abstract;

             properties = 'FILENAME';
             do while (rc = 0);
                CALL ENTRY_NEXT(packageId, entryId, entryType, userSpec, desc, nameValue, rc,
                                properties, filename);
                if (rc = 0) then do;
                   entryCount = entryCount + 1;
                   put 'Entry: ' entryCount;
                   put '        EntryId: '     entryId;
                   put '        Type: '        entryType;
                   put '        UserSpec: '    userSpec;
                   put '        Description: ' desc;
                   put '        nameValue: '   nameValue;

                   if (entryType = 'TEXT' OR entryType = 'BINARY') then do;
                      filepath = "FILENAME: &work_path/" || filename;
                      CALL RETRIEVE_FILE(entryId, filepath, rc2);
                      if (rc2 ne 0) then do;
                         msg = sysmsg();
                         put msg;
                     output;
                   call symput('fcf_sa_import_error_flg', '1');
                      end;
                      else do;
                         put 'File retrieved: ' filepath;
                      end;
                   end;
                end;
                else do;
                   /* warning that there are no more files */
                   msg = sysmsg();
                   put msg;
                end;
             end; /* do while */

             CALL PACKAGE_TERM(packageId, rc);
             if rc ne 0 then do;
                msg = sysmsg();
                put msg;
                output;
                call symput('fcf_sa_import_error_flg', '1');
             end;
           end;
       end;
      run;
      %if &fcf_sa_import_error_flg ne 0 %then %do;
              %let fcf_sa_import_err=%sysfunc(sasmsg(sashelp.fcfmsg, sa_import_err, N));
              %put ERROR: &fcf_sa_import_err;
              %fcf_sa_abort(msg=sa_import_err);
          %goto exit;
      %end;

      %if %sysfunc(fileexist("&work_path./fcf_export.xml")) ne 1 %then %do;
          %let fcf_sa_corrupted_package_err=%sysfunc(sasmsg(sashelp.cddmsg, sa_corrupted_package_err, N, fcf_export.xml));
          data errors1;
                  length msg $400;
                  msg = "&fcf_sa_corrupted_package_err";
                  output;
          run;
          proc append base=errors data=errors1 force;
          run;
          %let fcf_sa_import_error_flg = 1;
          %goto exit;
      %end;

      libname xmlin xml "&work_path./fcf_export.xml";

      proc copy in=xmlin out=&worklib. memtype=data ;
      run;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=&work_path./fcf_export.xml);

        /* fix datatypes, XML engine sets default types that do not match party_live */
      data &worklib..fcf_party_live;
             set &worklib..fcf_party_live (rename=(VALID_TO_DTTM=VALID_TO_DTTM_OLD INDIVIDUAL_FLG=INDIVIDUAL_FLG_OLD DELETE_FLG=DELETE_FLG_OLD IDENTICAL_PARTY_LINK_SK=IDENTICAL_PARTY_LINK_SK_OLD));
             drop VALID_TO_DTTM_OLD INDIVIDUAL_FLG_OLD DELETE_FLG_OLD IDENTICAL_PARTY_LINK_SK_OLD;

             length IDENTICAL_PARTY_LINK_SK VALID_TO_DTTM 8;

             if missing(IDENTICAL_PARTY_LINK_SK_OLD) then
                 IDENTICAL_PARTY_LINK_SK = .;
             else
                 IDENTICAL_PARTY_LINK_SK = input(IDENTICAL_PARTY_LINK_SK_OLD, BEST32. );

             VALID_TO_DTTM = .;

             if INDIVIDUAL_FLG_OLD = 1 then
                INDIVIDUAL_FLG = '1';
             else
                INDIVIDUAL_FLG = '0';

             if DELETE_FLG_OLD = 1 then
                DELETE_FLG = '1';
             else
                DELETE_FLG = '0';
      run;
  %end;

  %if %sysfunc(exist(&worklib..fcf_files)) %then %do;
      data &worklib..fcf_files;
            set &worklib..fcf_files;
            length NEW_FILE_NM $100;
            NEW_FILE_NM = FILE_NM;
      run;
  %end;

  %if &rename eq Y %then %do;
      /* rename scenarios - used for Copy */
      /* change PARTY_IDs for all parties, so that they are treated as new parties */
      data &worklib..fcf_party_live;
        set &worklib..fcf_party_live (rename=(party_rk=party_rk_old));
      run;
      /* get new PARTY_RK temporary to use them as new unique IDs */
      %fcf_get_ecm_sequence_ids(in=&worklib..fcf_party_live,out=&worklib..fcf_party_live,type=party,block=no);
      /* keep old party_rk, but rename all entities */
      data &worklib..fcf_party_live;
        set &worklib..fcf_party_live;
        length NEW_ID $32;
        length NEW_FILE_NM $100;
        NEW_ID=trim(put(PARTY_RK, 32.-L));
        if PARTY_CATEGORY_CD = 'SCENARIO' then do;
            PARTY_ID= 'SCR0' || trim(NEW_ID);
            NEW_FILE_NM = 'scr' || trim(NEW_ID) || '.sas';
        end;
        if PARTY_CATEGORY_CD = 'HEADER' then do;
            PARTY_ID= 'SCR0' || trim(NEW_ID);
            NEW_FILE_NM = 'hdr' || trim(NEW_ID) || '.sas';
        end;
        if PARTY_CATEGORY_CD = 'ROUTINGGRP' then do;
            PARTY_ID= 'RGR0' || trim(NEW_ID);
        end;
        PARTY_RK = party_rk_old;
        PARTY_FULL_NM = "Copy_" || trim(NEW_ID) || "_" || PARTY_FULL_NM;
        drop party_rk_old;
      run;
      /* update names stored as UDF values */
      proc sql;
        update &worklib..fcf_party_udf_char u
           set UDF_VALUE = (select PARTY_FULL_NM from &worklib..fcf_party_live p where u.party_rk = p.party_rk)
           where UDF_TABLE_NM = 'PARTY'
                 and (UDF_NM = 'SCENARIO_NAME' or UDF_NM = 'HEADER_NAME' or UDF_NM = 'ROUTING_GROUP_NAME')
        ;
      quit;

      /* assign new IDs for all entities to avoid clashes during deployment */
      proc sql;
        update &worklib..fcf_party_udf_char u
        set UDF_VALUE = (select NEW_ID from &worklib..fcf_party_live p where u.party_rk = p.party_rk)
        where UDF_NM = 'SCENARIO_ID' or UDF_NM = 'HEADER_ID' or UDF_NM = 'ROUTING_GROUP_ID'
        ;
      quit;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=fcf_party_udf_char_new_id);
      /* replace file names */
      proc sql;
        update &worklib..fcf_party_udf_char u
        set UDF_VALUE = (select NEW_FILE_NM  from &worklib..fcf_party_live p where u.party_rk = p.party_rk)
        where UDF_NM = 'SCENARIO_CODE_LOCATION_DESC' or UDF_NM = 'HEADER_LOCATION_DESC'
        ;
      quit;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=fcf_party_udf_char_new_file_nm);
      /* use new file names for upload */
      %if %sysfunc(exist(&worklib..fcf_files)) %then %do;
          proc sql;
            update &worklib..fcf_files f
              set NEW_FILE_NM = (select NEW_FILE_NM from &worklib..fcf_party_live p where f.party_rk = p.party_rk)
            ;
          quit;
          %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=fcf_files_new_file_nm);
      %end;
  %end;

  data _null_;
     call symput('current_dt',put(datetime(),datetime22.));
  run;

    /* split existing and new parties */

    /* select all SA entities from PARTY_LIVE to improve performance of joins later, this table will be quite small */
    proc sql;
      create table &worklib..party_live as
          select * from ecm_db.party_live p
          where   "&fcf_source_system_cd" = p.SOURCE_SYSTEM_CD and PARTY_TYPE_CD = "&fcf_sa_party_type"
          ;
    quit;

  proc sql;
    create table &worklib..p_live_new as
    select * from fcfimp.fcf_party_live p
    where  (select count(*) from &worklib..party_live e where p.party_id = e.party_id) = 0
    ;
  quit;
  %let new_count = &SQLOBS;
  %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=p_live_new);

  /* list of existing parties, party_rk from the target system should be preserved, version_no needs to be incremented */
  proc sql;
    create table &worklib..p_live_upd as
    select p.*, e.party_rk as party_rk_new, (e.version_no +1 ) as version_no_new
      from fcfimp.fcf_party_live p
      left join &worklib..party_live e
      on p.party_id = e.party_id and "&fcf_source_system_cd" = e.SOURCE_SYSTEM_CD
    where  not (p.party_rk in select party_rk from &worklib..p_live_new)
    ;
  quit;
  %let upd_count = &SQLOBS;
  %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=p_live_upd);

/* =================================================================================
   add new scenarios and groups - PARTY_RK has to be changed to use target system
   sequence to avoid duplicates in the future
  ================================================================================= */
  %if &new_count > 0 %then %do;

      data &worklib..p_live_new;
           length source_system_cd $10; /* need to override xml file length, it could be shorter than database */
           length update_user_id $60;
           set &worklib..p_live_new (rename=(party_rk=party_rk_old));

            source_system_cd = "&fcf_source_system_cd";
            version_no = 1;
            update_user_id = "&username";
            valid_from_dttm="&current_dt"dt;
            create_dttm="&current_dt"dt;
      run;

      %fcf_get_ecm_sequence_ids(in=&worklib..p_live_new,out=&worklib..p_live_new,type=party,block=no);
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=p_live_new_rename);

      proc append base=ecm_db.party_live data=&worklib..p_live_new force;
      run;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.party_live);
      proc append base=ecm_db.party_version data=&worklib..p_live_new force;
      run;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.party_version);

        /* add LOADEN events to fcf_EVENT */
      proc sql;
        create table &worklib..evt_new as
           select
              . as EVENT_RK                          length = 8 format = 11. informat = 11.,
              ("PARTY") as BUSINESS_OBJECT_NM        length = 100  format = $100. informat = $100.,
              PARTY_RK as BUSINESS_OBJECT_RK,
              ("LOADEN") as EVENT_TYPE_CD            length = 10 format = $10. informat = $10.,
              %if &packageType ne U %then %do;
                ("fcf.event.import.txt") as EVENT_DESC   length = 256 format = $256. informat = $256.,
              %end;
              %else %do;
                ("fcf.event.copy.txt") as EVENT_DESC   length = 256 format = $256. informat = $256.,
              %end;
              CREATE_USER_ID,
              CREATE_DTTM,
              "" as LINKED_OBJECT_NM                 length = 100 format = $100. informat = $100.,
              . as LINKED_OBJECT_RK                  length = 8 format = 11. informat = 11.
        from &worklib..p_live_new
        ;
      quit;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=evt_new);

      proc append base=&worklib..fcf_event data=&worklib..evt_new force;
      run;
      %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=evt_new_append);

        /* party_udf_def is relatively small table, using it in the join below
        * without significant negative effect on performance */
        /* add UDF char fields */
        %if %sysfunc(exist(&worklib..fcf_party_udf_char)) %then %do;
          proc sql;
            create table &worklib..p_udfc_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_char p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;

          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfc_new;
                     set &worklib..p_udfc_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                    valid_from_dttm="&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_CHAR_VALUE data=&worklib..p_udfc_new force;
            run;
            %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udf_char_value);
            %end;
        %end;

        %if %sysfunc(exist(&worklib..fcf_party_udf_date)) %then %do;
              /* add UDF date fields */
          proc sql;
            create table &worklib..p_udfd_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_date p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;

          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfd_new;
                     set &worklib..p_udfd_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                    valid_from_dttm="&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_DATE_VALUE data=&worklib..p_udfd_new force;
              run;
              %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udf_date_value);
          %end;
        %end;

        %if %sysfunc(exist(&worklib..fcf_party_udf_num)) %then %do;
              /* add UDF num fields */
          proc sql;
            create table &worklib..p_udfn_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_num p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;

          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfn_new;
                     set &worklib..p_udfn_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                     valid_from_dttm="&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_NUM_VALUE data=&worklib..p_udfn_new force;
              run;
              %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udf_num_value);
            %end;
        %end;

        %if %sysfunc(exist(&worklib..fcf_party_udf_lgchr)) %then %do;
              /* add UDF large char fields */
          proc sql;
            create table &worklib..p_udfl_new as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_lgchr p
              inner join &worklib..p_live_new k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfl_new;
                     set &worklib..p_udfl_new (rename=(party_rk=party_rk_old party_rk_new=party_rk));
                     drop party_rk_old;
                     valid_from_dttm="&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_LGCHR_VALUE data=&worklib..p_udfl_new force;
              run;
              %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udf_lgchr_value);
            %end;
        %end;

        /* update access in PARTY_X_USER_GROUP  */
        proc sql;
          create table &worklib..fcf_groups as
          select c.PARTY_TYPE_CD, c.PARTY_CATEGORY_CD, u.USER_GROUP_NM
          from ecm_db.party_config c
          join ecm_db.PARTY_CONFIG_X_USER_GROUP u
          on c.PARTY_CONFIG_SEQ_NO = u.PARTY_CONFIG_SEQ_NO
             and c.PARTY_TYPE_CD = "&fcf_sa_party_type"
             and (c.PARTY_CATEGORY_CD = "&fcf_sa_scenario_cat" or c.PARTY_CATEGORY_CD = "&fcf_sa_header_cat" or c.PARTY_CATEGORY_CD = "&fcf_sa_rg_cat")
          ;
        quit;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=fcf_groups);

        proc sql;
          insert into ecm_db.PARTY_X_USER_GROUP (PARTY_RK, USER_GROUP_NM)
            select p.PARTY_RK, g.USER_GROUP_NM
             from &worklib..p_live_new p
             join &worklib..fcf_groups g
             on p.PARTY_TYPE_CD = g.PARTY_TYPE_CD and p.PARTY_CATEGORY_CD = g.PARTY_CATEGORY_CD
             ;
        quit;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.PARTY_X_USER_GROUP);

      %if %sysfunc(exist(&worklib..fcf_files)) %then %do;
          /* upload files */
          proc sql;
            create table &worklib..fcf_files_new as
            select f.*, p.party_rk as party_rk_new
            from &worklib..fcf_files f
            join &worklib..p_live_new p
            on f.party_rk = p.party_rk_old
            ;
          run;

          %fcf_sa_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&fcf_scenarios_dav_path, category=SCENARIO,   ds=&worklib..fcf_files_new)
          %fcf_sa_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&fcf_headers_dav_path,   category=HEADER,     ds=&worklib..fcf_files_new)
     %end;
       /* party_x_party is processed after updated parties are processed so that we have all references */
  %end;

/* =================================================================================
   process existing scenarios, headers and rg -
   ECM existing PARTY_RK does not change in ECM
   ================================================================================= */
  %if &upd_count > 0 %then %do;
      /* use existing PARTY_RK */
        data &worklib..p_live_upd;
             set &worklib..p_live_upd (rename=(party_rk=party_rk_old party_rk_new=party_rk version_no=version_no_old version_no_new=version_no));
             drop version_no_old;
             VALID_FROM_DTTM = "&current_dt"dt;
             VALID_TO_DTTM = .;
             UPDATE_USER_ID = "&username";
        run;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=p_live_upd_rename);

        proc datasets lib = ecm_db nolist nowarn memtype = (data view);
           delete tmp_party_staging;
        quit;
        proc sql;
            create table ecm_db.tmp_party_staging like ecm_db.party_live;
        quit;
        proc append base=ecm_db.tmp_party_staging data=&worklib..p_live_upd FORCE;
        run;

        /* update existing records with new info, not all fields need to be updated */
        /* update SA parties in party_live */
        proc sql feedback;
            connect to &db_engine as &db_engine (authdomain=&ecm_dbauth &ecm_db_conopts.);
            execute (
                merge into party_live pl
                using (select party_rk, version_no, valid_from_dttm, party_full_nm, update_user_id from tmp_party_staging) tmp_party
                on (tmp_party.party_rk = pl.party_rk)
                when matched then update set
                    pl.VALID_FROM_DTTM=tmp_party.VALID_FROM_DTTM,
                    pl.PARTY_FULL_NM=tmp_party.PARTY_FULL_NM,
                    pl.UPDATE_USER_ID = tmp_party.UPDATE_USER_ID,
                    pl.VERSION_NO = tmp_party.VERSION_NO
                    %if %upcase(&db_engine) eq ODBC %then %do;
                        ;
                    %end;
            ) by &db_engine;
            disconnect from &db_engine;
        quit;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.party_live.update);
        /* mark existing version as expired */
        /* close party_version records for updated parties
         * cannot use pv.valid_to_dttm is null in the ON clause because Oracle does not allow
         * update columns that are used in the ON clause.
         * Instead using fact that pv.valid_to_dttm is null for the same records where
         * pv.version_no = tmp_party.version_no - 1
        */
        proc sql feedback;
            connect to &db_engine as &db_engine (authdomain=&ecm_dbauth &ecm_db_conopts.);
            execute (
                merge into party_version pv
                using (select party_rk, valid_from_dttm, version_no from tmp_party_staging) tmp_party
                on (tmp_party.party_rk = pv.party_rk and  pv.version_no = tmp_party.version_no - 1)
                when matched then update set
                    pv.VALID_TO_DTTM = tmp_party.VALID_FROM_DTTM
                    %if %upcase(&db_engine) eq ODBC %then %do;
            ;
                    %end;
                ) by &db_engine;
            disconnect from &db_engine;
       quit;
       %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.party_version);
        /* add new version to party_version*/
        proc append base=ecm_db.party_version data=&worklib..p_live_upd FORCE;
        run;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.party_live.append);
        /* add SAVENT events to fcf_EVENT */
       proc sql;
        create table &worklib..evt_upd as
           select
              . as EVENT_RK                     length = 8 format = 11. informat = 11.,
              ("PARTY") as BUSINESS_OBJECT_NM   length = 100 format = $100. informat = $100.,
              PARTY_RK as BUSINESS_OBJECT_RK,
              ("SAVENT") as EVENT_TYPE_CD       length = 10 format = $10. informat = $10.,
              ("fcf.event.import.version.fmt.txt" || "09"x || trim(left(put(VERSION_NO, BEST32.)))) as EVENT_DESC length = 256 format = $256. informat = $256.,
              "&username" as CREATE_USER_ID,
              "&current_dt"dt as CREATE_DTTM    length = 8 format = DATETIME27.6 informat = DATETIME27.6,
              "" as LINKED_OBJECT_NM            length = 100 format = $100. informat = $100.,
              . as LINKED_OBJECT_RK             length = 8format = 11. informat = 11.
           from &worklib..p_live_upd
        ;
       quit;
       %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=evt_upd);

       proc append base=&worklib..fcf_event data=&worklib..evt_upd force;
       run;
       %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=evt_new_append);

        /* update UDF char fields */
       %if %sysfunc(exist(&worklib..fcf_party_udf_char)) %then %do;
          proc sql;
            create table &worklib..p_udfc_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_char p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
            data &worklib..p_udfc_upd;
                     set &worklib..p_udfc_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
                     VALID_FROM_DTTM = "&current_dt"dt;
            run;
            proc append base=ecm_db.PARTY_UDF_CHAR_VALUE data=&worklib..p_udfc_upd force;
            run;
            %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udfc_upd);
         %end;
      %end;

        /* update UDF date fields */
        %if %sysfunc(exist(&worklib..fcf_party_udf_date)) %then %do;
          proc sql;
            create table &worklib..p_udfd_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_date p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfd_upd;
                     set &worklib..p_udfd_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
                     VALID_FROM_DTTM = "&current_dt"dt;
              run;
              proc append base=ecm_db.PARTY_UDF_DATE_VALUE data=&worklib..p_udfd_upd force;
              run;
              %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udfd_upd);
          %end;
        %end;

        /* update UDF num fields */
        %if %sysfunc(exist(&worklib..fcf_party_udf_num)) %then %do;
          proc sql;
            create table &worklib..p_udfn_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_num p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfn_upd;
                     set &worklib..p_udfn_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
                     VALID_FROM_DTTM = "&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_NUM_VALUE data=&worklib..p_udfn_upd force;
            run;
            %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udfc_upd);
          %end;
        %end;

            /* update UDF large char fields */
        %if %sysfunc(exist(&worklib..fcf_party_udf_lgchr)) %then %do;
          proc sql;
            create table &worklib..p_udfl_upd as
            select p.*, k.party_rk as PARTY_RK_NEW
              from &worklib..fcf_party_udf_lgchr p
              inner join &worklib..p_live_upd k
              on p.party_rk = k.party_rk_old
              inner join ecm_db.party_udf_def u
              on u.UDF_TABLE_NM = p.UDF_TABLE_NM and u.UDF_NM = p.UDF_NM
            ;
          quit;
          %if &SQLOBS > 0 %then %do;
              data &worklib..p_udfl_upd;
                     set &worklib..p_udfl_upd (rename=(party_rk=party_rk_old party_rk_new=PARTY_RK valid_from_dttm=valid_from_dttm_old));
                     drop party_rk_old valid_from_dttm_old;
                     VALID_FROM_DTTM = "&current_dt"dt;
              run;
            proc append base=ecm_db.PARTY_UDF_LGCHR_VALUE data=&worklib..p_udfl_upd force;
            run;
            %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_udfl_upd);
          %end;
        %end;

        /* no need to update PARTY_X_USER_GROUP  - not changing access on the target system */

       %if %sysfunc(exist(&worklib..fcf_files)) %then %do;
          /* upload files */
          proc sql;
              create table &worklib..fcf_files_upd as
              select f.*, p.party_rk as party_rk_new from &worklib..fcf_files f
              join &worklib..p_live_upd p
              on f.party_rk = p.party_rk_old
              ;
          quit;

          %fcf_sa_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&fcf_scenarios_dav_path, category=SCENARIO,   ds=&worklib..fcf_files_upd)
          %fcf_sa_upload_files(dav_server_root=&fcf_dav_root, dav_server_path=&fcf_headers_dav_path,   category=HEADER,     ds=&worklib..fcf_files_upd)
     %end;
  %end;

/* ================================================================================= */
/* process party_x_party                                                             */
/* ================================================================================= */

    %if %sysfunc(exist(&worklib..fcf_party_x_party)) %then %do;
        /* merge new and old parties back with corrected PARTY_ID */
        %if &upd_count > 0 %then %do;
            data &worklib..party_live_full_new;
                 set &worklib..p_live_upd;
            run;
            %if &new_count > 0 %then %do;
              proc append base=&worklib..party_live_full_new data= &worklib..p_live_new force;
              run;
            %end;
        %end;
        %else %do;
            data &worklib..party_live_full_new;
                 set &worklib..p_live_new;
        %end;

	    proc datasets lib = ecm_db nolist nowarn memtype = (data view);
	       delete tmp_party_staging;
	    quit;
	    proc sql;
	        create table ecm_db.tmp_party_staging like ecm_db.party_live;
	    quit;
	    proc append base=ecm_db.tmp_party_staging data=&worklib..party_live_full_new FORCE;
	    run;

        /* find scenarios that are already in a header or include RG and delete only those relationships
         * existing headers and rg relationships are preserved - they do not change,
         * for scenarios, all relationships are deleted and then re-created.
        */
        proc sql;
          create table &worklib..party_x_party_del as
             select * from ecm_db.party_x_party
             where member_party_rk in ( select party_rk from  ecm_db.tmp_party_staging f where f.PARTY_CATEGORY_CD='SCENARIO') /* select hdr_X_scr relationships */
                   or party_rk in ( select party_rk from  ecm_db.tmp_party_staging f where f.PARTY_CATEGORY_CD='SCENARIO') /* select scr_X_rg relationships */
          ;
        quit;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_x_party_del);
        /* use the same where clause to delete as in select above */
        proc sql;
          delete from ecm_db.party_x_party
             where member_party_rk in ( select party_rk from  ecm_db.tmp_party_staging f where f.PARTY_CATEGORY_CD='SCENARIO') /* select hdr_X_scr relationships */
                   or party_rk in ( select party_rk from  ecm_db.tmp_party_staging f where f.PARTY_CATEGORY_CD='SCENARIO') /* select scr_X_rg relationships */
          ;
        quit;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_x_party_del);

        /* create remove relationship events */
        proc sql;
            create table &worklib..evt_pxp_del as
               select
                  . as EVENT_RK                       length = 8 format = 11. informat = 11.,
                  ("PARTY") as BUSINESS_OBJECT_NM     length = 100 format = $100. informat = $100.,
                  PARTY_RK as BUSINESS_OBJECT_RK,
                  ("REMPAR") as EVENT_TYPE_CD         length = 10 format = $10. informat = $10.,
                  MEMBER_TYPE_CD as EVENT_DESC        length = 256 format = $256. informat = $256.,
                  "&username" as CREATE_USER_ID,
                  "&current_dt"dt as CREATE_DTTM      length= 8 format = DATETIME27.6 informat=DATETIME27.6,
                  "PARTY" as LINKED_OBJECT_NM         length = 100 format = $100. informat = $100.,
                  MEMBER_PARTY_RK as LINKED_OBJECT_RK length = 8 format = 11. informat = 11.
            from &worklib..party_x_party_del
            ;
        quit;
        proc append base=&worklib..fcf_event data=&worklib..evt_pxp_del force;
        run;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=fcf_event.append.evt_pxp_del);

        /* create new relationships, fcf_party_x_party contains only relationships for the scenarios included in the export */
        proc sql;
          create table &worklib..p_x_p_new as
          select x.*, parent.party_rk as party_rk_new, child.party_rk as member_party_rk_new
            from &worklib..fcf_party_x_party x
            inner join &worklib..party_live_full_new parent
            on parent.party_rk_old = x.party_rk
            inner join &worklib..party_live_full_new child
            on child.party_rk_old = x.member_party_rk
          ;
        quit;
        %let pxpNewCnt = &SQLOBS;
        %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=p_x_p_new);
        %if &pxpNewCnt > 0 %then %do;
            data &worklib..p_x_p_new;
                   set &worklib..p_x_p_new (rename=(party_rk=party_rk_old party_rk_new=party_rk member_party_rk=member_party_rk_old member_party_rk_new=member_party_rk));
                   drop party_rk_old member_party_rk_old;
            run;
        %end;

        %if %sysfunc(exist( &worklib..p_x_p_new )) and &pxpNewCnt > 0 %then %do;
          proc append base=ecm_db.party_x_party data=&worklib..p_x_p_new force;
          run;
          %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=party_x_party.append.p_x_p_new);
          /* create event records for added relationships*/
          proc sql;
              create table &worklib..evt_pxp_new as
                 select
                    . as EVENT_RK                       length = 8 format = 11. informat = 11.,
                    ("PARTY") as BUSINESS_OBJECT_NM     length = 100 format = $100. informat = $100.,
                    PARTY_RK as BUSINESS_OBJECT_RK,
                    ("ADDPAR") as EVENT_TYPE_CD         length = 10 format = $10. informat = $10.,
                    MEMBER_TYPE_CD as EVENT_DESC        length = 256 format = $256. informat = $256.,
                    "&username" as CREATE_USER_ID,
                    "&current_dt"dt as CREATE_DTTM      length= 8 format = DATETIME27.6 informat=DATETIME27.6,
                    "PARTY" as LINKED_OBJECT_NM         length = 100 format = $100. informat = $100.,
                    MEMBER_PARTY_RK as LINKED_OBJECT_RK length = 8 format = 11. informat = 11.
              from &worklib..p_x_p_new
              ;
          quit;
          %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=evt_pxp_new);
          proc append base=&worklib..fcf_event data=&worklib..evt_pxp_new force;
          run;
          %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=fcf_event_pxp.append.evt_pxp_new);
        %end;
    %end;

  /* update ECM_EVENT table */
  %fcf_get_ecm_sequence_ids(in=&worklib..fcf_event,out=&worklib..fcf_event,type=event,block=no);

  proc append base=ecm_db.ecm_event data=&worklib..fcf_event force;
  run;
  %fcf_sa_abort_on_error(msg=fcf_sa_processing_error, p1=ecm_db.ecm_event);

  /* in case of initial import during the deployment we must update localization data
   * using customers locale if it is not English
   */
  %if &packageType eq I and &fcf_config_locale ne en %then %do;
    /* update localization data, first parse the key and get the values using current server locale */
    data fcfsa_p;
        set sashelp.fcfsa;
        where locale="en"; /* use only en records bu UDF_VALUE will contain localized value */
        length party_id $64 type $32 row_no 8 UDF_VALUE $1000 row_no_str $10;
        keep party_id udf_value type row_no;
        type=kscan(key,1,'.');
        party_id=kscan(key,2,'.');
        row_no_str=kscan(key,3,'.');
        if row_no_str then row_no=input(row_no_str, BEST32.); else row_no = 1;
        UDF_VALUE=sasmsg("sashelp.fcfsa", key, 'NOQUOTE');
    run;
    /* get  party_rk */
    proc sql;
        create table fcfsa_u as
            select f.*, p.party_rk
            from fcfsa_p f
            inner join ecm_db.party_live p
                on  f.party_id = p.party_id and p.source_system_cd = "&fcf_source_system_cd"
        ;
    quit;

    /* update UDF fields for name and description for scenarios, parameters and headers */
    %fcf_sa_update_udf_fields (src_type=HDESC, udf_nm=HEADER_DESCRIPTION, udf_table_nm=PARTY);
    %fcf_sa_update_udf_fields (src_type=SSDESC, udf_nm=SCENARIO_SHORT_DESCRIPTION, udf_table_nm=PARTY);
    %fcf_sa_update_udf_fields (src_type=SDESC, udf_nm=SCENARIO_DESCRIPTION, udf_table_nm=PARTY);
    %fcf_sa_update_udf_fields (src_type=SPDESC, udf_nm=PARM_DESC, udf_table_nm=X_SCR_P);
  %end;

%exit:
%mend fcf_sa_import_scenarios;
